# -*- coding: utf-8 -*-

import itertools

a2i = [
    "arn:aws:sagemaker:us-east-1:111122223333:flow-definition/my_flow",
    "arn:aws:sagemaker:us-east-1:111122223333:human-loop/1a2b3c",
    "arn:aws:sagemaker:us-east-1:111122223333:human-task-ui/my-ui",
]
apigateway = [
    "arn:aws:apigateway:us-east-1::/restapis/my_rest_api/stages/my_stage/path/to/resource",
    "arn:aws:apigateway:us-east-1::/restapis/my_rest_api/deployments/my_deployment",
    "arn:aws:apigateway:us-east-1::/restapis/my_rest_api/authorizers/my_authorizer",
    "arn:aws:apigateway:us-east-1::/restapis/my_rest_api/models/my_model",
    "arn:aws:apigateway:us-east-1::/restapis/my_api/routes/my_route",
    "arn:aws:apigateway:us-east-1::/restapis/my_api/integrations/my_integration",
    "arn:aws:apigateway:us-east-1::/apis/my_api/stages/my_stage/path/to/resource",
    "arn:aws:apigateway:us-east-1::/apis/my_api/deployments/my_deployment",
    "arn:aws:apigateway:us-east-1::/apis/my_api/authorizers/my_authorizer",
    "arn:aws:apigateway:us-east-1::/apis/my_api/models/my_model",
    "arn:aws:apigateway:us-east-1::/apis/my_api/routes/my_route",
    "arn:aws:apigateway:us-east-1::/apis/my_api/integrations/my_integration",
]
batch = [
    "arn:aws:batch:us-east-1:111122223333:compute-environment/test",
    "arn:aws:batch:us-east-1:111122223333:job-queue/test",
    "arn:aws:batch:us-east-1:111122223333:job-definition/test:1",
    "arn:aws:batch:us-east-1:111122223333:job/b2957570-6bae-47b1-a2d8-af4f3030fc36",
]
cloudformation = [
    "arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/1a2b3c",
    "arn:aws:cloudformation:us-east-1:111122223333:changeSet/my-stack-name-2000-01-01/1a2b3c",
    "arn:aws:cloudformation:us-east-1:111122223333:stackset/my-stack-set:1a2b3c",
]
cloudwatch_logs = [
    "arn:aws:logs:us-east-1:111122223333:log-group:/aws/lambda/my-func:*",
    "arn:aws:logs:us-east-1:111122223333:log-group:my-log-group*:log-stream:my-log-stream*",
]
codecommit = [
    "arn:aws:codecommit:us-east-1:111122223333:test",
    "arn:aws:codebuild:us-east-1:111122223333:project/test",
    "arn:aws:codebuild:us-east-1:111122223333:build/test:08805851-8a0a-4968-9d08-c7cc0623db7b",
    "arn:aws:codebuild:us-east-1:111122223333:build-batch/test:08805851-8a0a-4968-9d08-c7cc0623db7b",
    "arn:aws:codepipeline:us-east-1:111122223333:test",
]
dynamodb = [
    "arn:aws:dynamodb:us-east-1:111122223333:table/my_table",
    "arn:aws:dynamodb::111122223333:global-table/my_global_table_name",
    "arn:aws:dynamodb:us-east-1:111122223333:table/my_table/index/my_index",
    "arn:aws:dynamodb:us-east-1:111122223333:table/my_table/stream/my_stream_label",
    "arn:aws:dynamodb:us-east-1:111122223333:table/my_table/backup/my_backup_name",
    "arn:aws:dynamodb:us-east-1:111122223333:table/my_table/export/my_export_name",
    "arn:aws:dynamodb:us-east-1:111122223333:table/my_table/import/my_import_name",
]
ec2 = [
    "arn:aws:ec2:us-east-1:111122223333:instance/*",
    "arn:aws:ec2:us-east-1:123456789012:instance/i-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:key-pair/key-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:volume/vol-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:snapshot/snap-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:network-interface/eni-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:vpc/vpc-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:subnet/subnet-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:route-table/rtb-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:internet-gateway/igw-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:natgateway/nat-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:dhcp-options/dopt-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:vpc-peering-connection/pcx-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:network-acl/acl-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:security-group/sg-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:security-group-rule/sgr-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:vpc-endpoint/vpce-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:ipv4pool-ec2/eipalloc-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:customer-gateway/cgw-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:vpn-gateway/vgw-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:vpn-connection/vpn-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:client-vpn-endpoint/cvpn-endpoint-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:transit-gateway/tgw-1234567890abcdef0",
    "arn:aws:ec2:us-east-1:123456789012:transit-gateway-attachment/tgw-attach-1234567890abcdef0",
    "arn:aws:ec2:us-east-1::image/ami-1234567890abcdef0",
]
ecr = [
    "arn:aws:ecr:us-east-1:123456789012:repository/my-repo",
]
ecs = [
    "arn:aws:ecs:us-east-1:111122223333:cluster/my-cluster-1",
    "arn:aws:ecs:us-east-1:111122223333:task-definition/my-task:1",
    "arn:aws:ecs:us-east-1:111122223333:container-instance/my-cluster/container_instance_UUID",
    "arn:aws:ecs:us-east-1:111122223333:service/service_name",
    "arn:aws:ecs:us-east-1:111122223333:task/my_cluster/a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE"
]
glue = [
    "arn:aws:glue:us-east-1:111122223333:catalog",
    "arn:aws:glue:us-east-1:111122223333:database/db1",
    "arn:aws:glue:us-east-1:111122223333:table/db1/tbl",
    "arn:aws:glue:us-east-1:111122223333:crawler/mycrawler",
    "arn:aws:glue:us-east-1:111122223333:job/testjob",
    "arn:aws:glue:us-east-1:111122223333:trigger/sampletrigger",
    "arn:aws:glue:us-east-1:111122223333:devEndpoint/temporarydevendpoint",
    "arn:aws:glue:us-east-1:111122223333:mlTransform/tfm-1234567890",
]
kinesis = [
    "arn:aws:kinesisvideo:us-east-1:111122223333:stream/kinesis-stream-name/111122223333",
]
kms = [
    "arn:aws:kms:us-east-1:111122223333:key/1a2b3c",
    "arn:aws:kms:us-east-1:111122223333:alias/my_key",
]
lambda_func = [
    "arn:aws:lambda:us-east-1:111122223333:function:my-func",
    "arn:aws:lambda:us-east-1:111122223333:function:my-func:LIVE",
    "arn:aws:lambda:us-east-1:111122223333:function:my-func:1",
    "arn:aws:lambda:us-east-1:111122223333:layer:my-layer:1",
]
macie = [
    "arn:aws:macie:us-east-1:111122223333:trigger/example0954663fda0f652e304dcc21323508db/alert/example09214d3e70fb6092cc93cee96dbc4de6",
]
opensearch = [
    "arn:aws:es:us-east-1:111122223333:domain/my_domain",
]
opensearch_serverless = [
    "arn:aws:aoss:us-east-1:111122223333:collection/collection_id",
    "arn:aws:aoss:us-east-1:111122223333:dashboards/default",
]
rds = [
    "arn:aws:rds:us-east-1:111122223333:db:my-mysql-instance-1",
    "arn:aws:rds:us-east-1:111122223333:cluster:my-aurora-cluster-1",
    "arn:aws:rds:us-east-1:111122223333:es:my-subscription",
    "arn:aws:rds:us-east-1:111122223333:og:my-og",
    "arn:aws:rds:us-east-1:123456789012:pg:my-param-enable-logs",
    "arn:aws:rds:us-east-1:111122223333:cluster-pg:my-cluster-param-timezone",
    "arn:aws:rds:us-east-1:111122223333:ri:my-reserved-postgresql",
    "arn:aws:rds:us-east-1:111122223333:secgrp:my-public",
    "arn:aws:rds:us-east-1:111122223333:snapshot:rds:my-mysql-db-2020-01-01-00-00",
    "arn:aws:rds:us-east-1:111122223333:cluster-snapshot:rds:my-aurora-cluster-2020-01-01-00-00",
    "arn:aws:rds:us-east-1:111122223333:snapshot:my-mysql-db-snap",
    "arn:aws:rds:us-east-1:111122223333:cluster-snapshot:my-aurora-cluster-snap",
    "arn:aws:rds:us-east-1:111122223333:subgrp:my-subnet-10",
]
redshift = [
    "arn:aws:redshift:us-east-1:111122223333:cluster:my_cluster",
    "arn:aws:redshift:us-east-1:111122223333:dbgroup:my_cluster/my_db_group",
    "arn:aws:redshift:us-east-1:111122223333:dbname:my_cluster/my_database",
    "arn:aws:redshift:us-east-1:111122223333:snapshot:my_cluster/my_snapshot",
    "arn:aws:redshift:us-east-1:111122223333:snapshotschedule:my_snapshot_schedule",
    "arn:aws:redshift:us-east-1:111122223333:parametergroup:my_parameter_group",
    "arn:aws:redshift:us-east-1:111122223333:subnetgroup:my_subnet_group",
    "arn:aws:redshift:us-east-1:111122223333:securitygroup:my_group_name/ec2securitygroup/owner_name/sg-1a2b",
]
redshift_serverless = [
    "arn:aws:redshift-serverless:us-east-1:111122223333:namespace/my_namespace",
    "arn:aws:redshift-serverless:us-east-1:111122223333:workgroup/my_workgroup",
    "arn:aws:redshift-serverless:us-east-1:111122223333:snapshot/my_snapshot",
    "arn:aws:redshift-serverless:us-east-1:111122223333:managedvpcendpoint/my_vpc_endpoint",
]
s3 = [
    "arn:aws:s3:::my-bucket",
    "arn:aws:s3:::my-bucket/cloudformation/upload/10f3db7bcfa62c69e5a71fef595fac84.json",
]
sagemaker = [
    "arn:aws:sagemaker:us-east-1:111122223333:action/my_action",
    "arn:aws:sagemaker:us-east-1:111122223333:algorithm/my_algorithm",
    "arn:aws:sagemaker:us-east-1:111122223333:app-image-config/my_app_image_config",
    "arn:aws:sagemaker:us-east-1:111122223333:automl-job/my_automl_job",
    "arn:aws:sagemaker:us-east-1:111122223333:code-repository/my_code_repository",
    "arn:aws:sagemaker:us-east-1:111122223333:compilation-job/my_compilation_job",
    "arn:aws:sagemaker:us-east-1:111122223333:context/my_context",
    "arn:aws:sagemaker:us-east-1:111122223333:data-quality-job-definition/my_data_quality_job_definition",
    "arn:aws:sagemaker:us-east-1:111122223333:endpoint/my_endpoint",
    "arn:aws:sagemaker:us-east-1:111122223333:endpoint-config/my_endpoint_config",
    "arn:aws:sagemaker:us-east-1:111122223333:experiment/my_experiment",
    "arn:aws:sagemaker:us-east-1:111122223333:experiment-trial/",
    "arn:aws:sagemaker:us-east-1:111122223333:experiment-trial-component/",
    "arn:aws:sagemaker:us-east-1:111122223333:feature-group/",
    "arn:aws:sagemaker:us-east-1:111122223333:hub/my_hub",
    "arn:aws:sagemaker:us-east-1:111122223333:hub-content/my_hub/my_hub_content_type/my_hub_content_name",
    "arn:aws:sagemaker:us-east-1:111122223333:hyper-parameter-tuning-job/my_hyper_parameter_tuning_job",
    "arn:aws:sagemaker:us-east-1:111122223333:image/my_image",
    "arn:aws:sagemaker:us-east-1:111122223333:image-version/my_image/1",
    "arn:aws:sagemaker:us-east-1:111122223333:inference-experiment/my_inference_experiment",
    "arn:aws:sagemaker:us-east-1:111122223333:inference-recommendations-job/my_inference_recommendations_job",
    "arn:aws:sagemaker:us-east-1:111122223333:labeling-job/my_labeling_job",
    "arn:aws:sagemaker:us-east-1:111122223333:model/my_model",
    "arn:aws:sagemaker:us-east-1:111122223333:model-bias-job-definition/my_model_bias_job_definition",
    "arn:aws:sagemaker:us-east-1:111122223333:model-card/my_model_card",
    "arn:aws:sagemaker:us-east-1:111122223333:model-card/my_model_card/export-job/my_export_job_name",
    "arn:aws:sagemaker:us-east-1:111122223333:model-explainability-job-definition/my_model_explainability_job_definition",
    "arn:aws:sagemaker:us-east-1:111122223333:model-package/my_model_package",
    "arn:aws:sagemaker:us-east-1:111122223333:model-package-group/my_model_package_group",
    "arn:aws:sagemaker:us-east-1:111122223333:model-quality-job-definition/my_model_quality_job_definition",
    "arn:aws:sagemaker:us-east-1:111122223333:monitoring-schedule/my_monitoring_schedule",
    "arn:aws:sagemaker:us-east-1:111122223333:monitoring-schedule/my_monitoring_schedule/alert/my_monitoring_schedule_alert_name",
    "arn:aws:sagemaker:us-east-1:111122223333:notebook-instance/my_notebook_instance",
    "arn:aws:sagemaker:us-east-1:111122223333:pipeline/my_pipeline",
    "arn:aws:sagemaker:us-east-1:111122223333:pipeline/my_pipeline/execution/1a2b3c",
    "arn:aws:sagemaker:us-east-1:111122223333:processing-job/my_processing_job",
    "arn:aws:sagemaker:us-east-1:111122223333:shared-model/share_model_id",
    "arn:aws:sagemaker:us-east-1:111122223333:shared-model-event/share_model_event_id",
    "arn:aws:sagemaker:us-east-1:111122223333:training-job/my_training_job",
    "arn:aws:sagemaker:us-east-1:111122223333:transform-job/my_transform_job",
    "arn:aws:sagemaker:us-east-1:111122223333:workforce/my_workforce",
    "arn:aws:sagemaker:us-east-1:111122223333:workteam/my_workteam",
    "arn:aws:sagemaker:us-east-1:111122223333:domain/domain_id",
    "arn:aws:sagemaker:us-east-1:111122223333:user-profile/domain_id/my_user_profile",
    "arn:aws:sagemaker:us-east-1:111122223333:space/domain_id/my_space",
    "arn:aws:sagemaker:us-east-1:111122223333:app/domain_id/my_user_profile/my_app_type/my_app_name",
    "arn:aws:sagemaker:us-east-1:111122223333:studio-lifecycle-config/my_studio_lifecycle_config",
    "arn:aws:sagemaker:us-east-1:111122223333:device-fleet/fleet_name/device/device_name",
    "arn:aws:sagemaker:us-east-1:111122223333:device-fleet/fleet_name",
]
secretmanager = [
    "arn:aws:secretsmanager:us-east-1:111122223333:secret:MyFolder/MySecret-a1b2c3",
]
sfn = [
    "arn:aws:states:us-east-1:111122223333:stateMachine:standard_test",
    "arn:aws:states:us-east-1:111122223333:stateMachine:express_test",
    "arn:aws:states:us-east-1:807388292768:stateMachine:standard_test:1",
    "arn:aws:states:us-east-1:807388292768:stateMachine:standard_test:LIVE",
    "arn:aws:states:us-east-1:111122223333:execution:standard_test:1d858cf6-613f-4576-b94f-e0d654c23843",
    "arn:aws:states:us-east-1:111122223333:express:express_test:e935dec6-e748-4977-a2f2-32eeb83d81da:b2f7726e-9b98-4a49-a6c4-9cf23a61f180",
]
sns = [
    "arn:aws:sns:us-east-1:111122223333:my_topic",
    "arn:aws:sns:us-east-1:111122223333:my_topic:a07e1034-10c0-47a6-83c2-552cfcca42db",
]
sqs = [
    "arn:aws:sqs:us-east-1:111122223333:my_queue",
]
ssm = [
    "arn:aws:ssm:us-east-1:111122223333:parameter/my_param",
    "arn:aws:ssm:us-east-1:111122223333:parameter/path/to/my_param",
]

arns = list(
    itertools.chain(
        a2i,
        apigateway,
        batch,
        cloudformation,
        cloudwatch_logs,
        codecommit,
        dynamodb,
        ec2,
        ecr,
        ecs,
        glue,
        kinesis,
        kms,
        lambda_func,
        macie,
        opensearch,
        opensearch_serverless,
        rds,
        redshift,
        redshift_serverless,
        s3,
        sagemaker,
        secretmanager,
        sfn,
        sns,
        sqs,
        ssm,
    )
)



# -*- coding: utf-8 -*-

"""
.. note::

    Let's use this as an example to show how to create a module for specific AWS services.

    First, you need to create an aws service class that the name is the service,
    and the base class is one of the following:

    - CrossAccountGlobal
    - Global
    - Regional
    - ResourceIdOnlyRegional
    - ColonSeparatedRegional
    - SlashSeparatedRegional

    Then all aws resources in this service should be a class that inherits from
    the service class.
"""

import typing as T
import dataclasses

from ..model import _CrossAccountGlobal


@dataclasses.dataclass
class S3(_CrossAccountGlobal):
    service: str = dataclasses.field(default="s3")
    resource_type: str = dataclasses.field(default=None)
    sep: str = dataclasses.field(default=None)


@dataclasses.dataclass
class S3Bucket(S3):
    """
    Example: arn:aws:s3:::my-bucket
    """
    @property
    def bucket_name(self) -> str:
        return self.resource_id

    @property
    def uri(self) -> str:
        return f"s3://{self.bucket_name}"

    @classmethod
    def new(
        cls,
        bucket_name: str,
    ):
        return cls(
            resource_id=bucket_name,
        )

    @classmethod
    def from_uri(cls, uri: str):
        return cls.new(uri.split("/")[2])


@dataclasses.dataclass
class S3Object(S3):
    """
    Example: arn:aws:s3:::my-bucket/folder/file.txt
    """
    @property
    def bucket(self) -> str:
        return self.resource_id.split("/", 1)[0]

    @property
    def key(self) -> str:
        return self.resource_id.split("/", 1)[1]

    @property
    def uri(self) -> str:
        return f"s3://{self.bucket}/{self.key}"

    @classmethod
    def new(cls, bucket: str, key: str):
        return cls(
            resource_id=f"{bucket}/{key}",
        )

    @classmethod
    def from_uri(cls, uri: str):
        parts = uri.split("/", 3)
        return cls.new(bucket=parts[2], key=parts[3])



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import typing as T
import dataclasses

from ..model import _ColonSeparatedRegional


@dataclasses.dataclass
class AwsLambda(_ColonSeparatedRegional):
    service: str = dataclasses.field(default="lambda")


@dataclasses.dataclass
class LambdaFunction(AwsLambda):
    """
    Example:

    - arn:aws:lambda:us-east-1:111122223333:function:my-func
    - arn:aws:lambda:us-east-1:111122223333:function:my-func:LIVE
    - arn:aws:lambda:us-east-1:111122223333:function:my-func:1
    """

    resource_type: str = dataclasses.field(default="function")

    @property
    def name(self) -> str:
        return self.resource_id.split(":", 1)[0]

    @property
    def function_name(self) -> str:
        return self.name

    @property
    def version(self) -> T.Optional[str]:
        words = self.resource_id.split(":", 1)
        if len(words) == 2:
            token = words[1]
            if token == "$LATEST":  # pragma: no cover
                return token
            elif token.isdigit():
                return token
            else:
                raise ValueError(f"Invalid version: {token}")
        else:
            return None

    @property
    def alias(self) -> T.Optional[str]:
        words = self.resource_id.split(":", 1)
        if len(words) == 2:
            token = words[1]
            if token == "$LATEST":  # pragma: no cover
                raise ValueError("Cannot specify alias for $LATEST")
            elif token.isdigit():
                raise ValueError(f"Invalid version: {token}")
            else:
                return token
        else:
            return None

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
        version: T.Optional[T.Union[str, int]] = None,
        alias: T.Optional[str] = None,
    ):
        """
        Factory method.
        """
        if version is not None and alias is not None:  # pragma: no cover
            raise ValueError("Cannot specify both version and alias")

        if version is not None:
            resource_id = f"{name}:{version}"
        elif alias is not None:
            resource_id = f"{name}:{alias}"
        else:
            resource_id = name

        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=resource_id,
        )


@dataclasses.dataclass
class LambdaLayer(AwsLambda):
    """
    Example: arn:aws:lambda:us-east-1:111122223333:layer:my-layer:1
    """

    resource_type: str = dataclasses.field(default="layer")

    @property
    def name(self) -> str:
        return self.resource_id.split(":", 1)[0]

    @property
    def layer_name(self) -> str:
        return self.name

    @property
    def version(self) -> int:
        return int(self.resource_id.split(":", 1)[1])

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
        version: int,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{name}:{version}",
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class Dynamodb(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="dynamodb")


@dataclasses.dataclass
class DynamodbTable(Dynamodb):
    """
    Example: arn:aws:dynamodb:us-east-1:111122223333:table/my_table
    """

    resource_type: str = dataclasses.field(default="table")

    @property
    def table_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
    ):
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=table_name,
        )


@dataclasses.dataclass
class DynamodbGlobalTable(Dynamodb):
    """
    Example: arn:aws:dynamodb::111122223333:global-table/my_global_table_name
    """

    region: str = dataclasses.field(default=None)
    resource_type: str = dataclasses.field(default="global-table")

    @property
    def table_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        table_name: str,
    ):
        return cls(
            account_id=aws_account_id,
            resource_id=table_name,
        )


@dataclasses.dataclass
class _DynamodbTableCommon(Dynamodb):
    """
    todo: docstring
    """

    resource_type: str = dataclasses.field(default="table")

    @property
    def table_name(self) -> str:
        return self.resource_id.split("/")[0]

    @property
    def table_resource_type(self) -> str:
        return self.resource_id.split("/")[1]

    @property
    def table_resource_name(self) -> str:
        return "/".join(self.resource_id.split("/")[2:])

    @classmethod
    def _new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
        resource_type: str,
        resource_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{table_name}/{resource_type}/{resource_name}",
        )


@dataclasses.dataclass
class DynamodbTableIndex(_DynamodbTableCommon):
    """
    Example: arn:aws:dynamodb:us-east-1:111122223333:table/my_table/index/my_index
    """

    @property
    def index_name(self) -> str:  # pragma: no cover
        return self.table_resource_name

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
        index_name: str,
    ):  # pragma: no cover
        return cls._new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            table_name=table_name,
            resource_type="index",
            resource_name=index_name,
        )


@dataclasses.dataclass
class DynamodbTableStream(_DynamodbTableCommon):
    """
    Example: arn:aws:dynamodb:us-east-1:111122223333:table/my_table/stream/my_stream_label
    """

    @property
    def stream_name(self) -> str:  # pragma: no cover
        return self.table_resource_name

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
        stream_name: str,
    ):  # pragma: no cover
        return cls._new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            table_name=table_name,
            resource_type="stream",
            resource_name=stream_name,
        )


@dataclasses.dataclass
class DynamodbTableBackup(_DynamodbTableCommon):
    """
    Example: arn:aws:dynamodb:us-east-1:111122223333:table/my_table/backup/my_backup_name
    """

    @property
    def backup_name(self) -> str:  # pragma: no cover
        return self.table_resource_name

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
        resource_name: str,
    ):  # pragma: no cover
        return cls._new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            table_name=table_name,
            resource_type="backup",
            resource_name=resource_name,
        )


@dataclasses.dataclass
class DynamodbTableExport(_DynamodbTableCommon):
    """
    Example: arn:aws:dynamodb:us-east-1:111122223333:table/my_table/export/my_export_name
    """

    @property
    def export_name(self) -> str:  # pragma: no cover
        return self.table_resource_name

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
        export_name: str,
    ):  # pragma: no cover
        return cls._new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            table_name=table_name,
            resource_type="export",
            resource_name=export_name,
        )


@dataclasses.dataclass
class DynamodbTableImport(_DynamodbTableCommon):
    """
    Example: arn:aws:dynamodb:us-east-1:111122223333:table/my_table/import/my_import_name
    """

    @property
    def import_name(self) -> str:  # pragma: no cover
        return self.table_resource_name

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        table_name: str,
        import_name: str,
    ):  # pragma: no cover
        return cls._new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            table_name=table_name,
            resource_type="import",
            resource_name=import_name,
        )



# -*- coding: utf-8 -*-

"""
Usage example::

    import aws_arns.api as aws_arns
"""

from .constants import AwsPartitionEnum
from .model import BaseArn
from .model import Arn
from .model import is_arn_instance
from . import resource as res
from .parse_arn import parse_arn



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class Kms(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="kms")


@dataclasses.dataclass
class KmsKey(Kms):
    """
    Example: "arn:aws:kms:us-east-1:111122223333:key/1a2b3c"
    """

    resource_type: str = dataclasses.field(default="key")

    @property
    def key_id(self) -> str:
        """
        "1a2b3c" part of
        arn:aws:kms:us-east-1:111122223333:key/1a2b3c
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        key_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=key_id,
        )


@dataclasses.dataclass
class KmsAlias(Kms):
    """
    Example: "arn:aws:kms:us-east-1:111122223333:alias/my_key"
    """

    resource_type: str = dataclasses.field(default="alias")

    @property
    def alias(self) -> str:
        """
        "my_key" part of
        arn:aws:kms:us-east-1:111122223333:alias/my_key
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        alias: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=alias,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _ResourceIdOnlyRegional


@dataclasses.dataclass
class Sns(_ResourceIdOnlyRegional):
    service: str = dataclasses.field(default="sns")


@dataclasses.dataclass
class SnsTopic(Sns):
    """
    Example: arn:aws:s3:::my-bucket
    """

    @property
    def topic_name(self) -> str:
        """
        SNS topic name
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        topic_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=topic_name,
        )


@dataclasses.dataclass
class SnsSubscription(Sns):
    """
    Example: arn:aws:s3:::my-bucket
    """

    @property
    def topic_name(self) -> str:
        return self.resource_id.split(":", 1)[0]

    @property
    def subscription_id(self) -> str:
        return self.resource_id.split(":", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        topic_name: str,
        subscription_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{topic_name}:{subscription_id}",
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class Ecr(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="ecr")


@dataclasses.dataclass
class EcrRepository(Ecr):
    """
    Example: arn:aws:ecr:us-east-1:123456789012:repository/my-repo
    """

    resource_type: str = dataclasses.field(default="repository")

    @property
    def repo_name(self) -> str:
        """
        The "my-repo" part of
        arn:aws:ecr:us-east-1:123456789012:repository/my-repo
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        repo_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=repo_name,
        )

    @property
    def uri(self) -> str:
        return f"{self.aws_account_id}.dkr.ecr.{self.aws_region}.amazonaws.com/{self.repo_name}"

    @classmethod
    def from_uri(cls, uri: str):
        """
        Factory method.
        """
        domain, name = uri.split("/", 1)
        parts = domain.split(".")
        aws_account_id = parts[0]
        aws_region = parts[3]
        return cls.new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            repo_name=name,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _ColonSeparatedRegional, _SlashSeparatedRegional


@dataclasses.dataclass
class Redshift(_ColonSeparatedRegional):
    service: str = dataclasses.field(default="redshift")


@dataclasses.dataclass
class RedshiftServerless(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="redshift-serverless")


@dataclasses.dataclass
class _RedshiftCommon(Redshift):
    """
    todo: docstring
    """


@dataclasses.dataclass
class RedshiftCluster(_RedshiftCommon):
    """
    Example: arn:aws:redshift:us-east-1:414570653400:cluster:my_cluster
    """

    resource_type: str = dataclasses.field(default="cluster")

    @property
    def cluster_id(self) -> str:
        """
        "my_cluster" part of
        arn:aws:redshift:us-east-1:414570653400:cluster:my_cluster
        """

        return self.resource_id


@dataclasses.dataclass
class _RedshiftClusterCommon(Redshift):
    """
    todo: docstring
    """

    @property
    def cluster_id(self) -> str:
        return self.resource_id.split("/", 1)[0]

    @property
    def resource_name(self) -> str:
        return self.resource_id.split("/", 1)[1]


@dataclasses.dataclass
class RedshiftDatabaseUserGroup(_RedshiftClusterCommon):
    """
    Example: arn:aws:redshift:us-east-1:414570653400:dbgroup:my_cluster/my_db_group
    """

    resource_type: str = dataclasses.field(default="dbgroup")

    @property
    def user_group(self) -> str:
        return self.resource_id.split("/", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        cluster_name: str,
        user_group: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{cluster_name}/{user_group}",
        )


@dataclasses.dataclass
class RedshiftDatabaseName(_RedshiftClusterCommon):
    """
    Example: arn:aws:redshift:us-east-1:414570653400:dbname:my_cluster/my_database
    """

    resource_type: str = dataclasses.field(default="dbname")

    @property
    def database_name(self) -> str:
        return self.resource_id.split("/", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        cluster_name: str,
        database_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{cluster_name}/{database_name}",
        )


@dataclasses.dataclass
class RedshiftSnapshot(_RedshiftClusterCommon):
    """
    Example: arn:aws:redshift:us-east-1:414570653400:snapshot:my_cluster/my_snapshot
    """

    resource_type: str = dataclasses.field(default="snapshot")

    @property
    def snapshot_name(self) -> str:
        return self.resource_id.split("/", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        cluster_name: str,
        snapshot_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{cluster_name}/{snapshot_name}",
        )


@dataclasses.dataclass
class RedshiftSnapshotSchedule(_RedshiftCommon):
    """
    Example: arn:aws:redshift:us-east-1:414570653400:snapshotschedule:my_snapshot_schedule
    """

    resource_type: str = dataclasses.field(default="snapshotschedule")

    @property
    def snapshot_schedule_name(self) -> str:
        return self.resource_id


@dataclasses.dataclass
class RedshiftParameterGroup(_RedshiftCommon):
    """
    Example: arn:aws:redshift:us-east-1:414570653400:parametergroup:my_parameter_group
    """

    resource_type: str = dataclasses.field(default="parametergroup")

    @property
    def parameter_group_name(self) -> str:
        return self.resource_id


@dataclasses.dataclass
class RedshiftSubnetGroup(_RedshiftCommon):
    """
    Example: arn:aws:redshift:us-east-1:111122223333:subnetgroup:my_subnet_group
    """

    resource_type: str = dataclasses.field(default="subnetgroup")

    @property
    def subnet_group_name(self) -> str:
        return self.resource_id


@dataclasses.dataclass
class RedshiftSecurityGroup(Redshift):
    """
    Example: arn:aws:redshift:us-east-1:111122223333:securitygroup:my_group_name/ec2securitygroup/owner_name/sg-1a2b
    """

    resource_type: str = dataclasses.field(default="securitygroup")

    @property
    def security_group_name(self) -> str:
        return self.resource_id.split("/", 3)[0]

    @property
    def owner_name(self) -> str:
        return self.resource_id.split("/", 3)[2]

    @property
    def ec2_security_group_id(self) -> str:
        return self.resource_id.split("/", 3)[3]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        security_group_name: str,
        owner_name: str,
        ec2_security_group_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{security_group_name}/ec2securitygroup/{owner_name}/{ec2_security_group_id}",
        )


@dataclasses.dataclass
class _RedshiftServerlessCommon(RedshiftServerless):
    """
    todo: docstring
    """

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        resource_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=resource_id,
        )


@dataclasses.dataclass
class RedshiftServerlessNamespace(_RedshiftServerlessCommon):
    """
    Example: arn:aws:redshift-serverless:us-east-1:111122223333:namespace/my_namespace
    """

    resource_type: str = dataclasses.field(default="namespace")

    @property
    def namespace_id(self) -> str:
        return self.resource_id

    @property
    def namespace_name(self):  # pragma: no cover
        raise AttributeError("You cannot get the human friendly name from the ARN!")


@dataclasses.dataclass
class RedshiftServerlessWorkgroup(_RedshiftServerlessCommon):
    """
    Example: arn:aws:redshift-serverless:us-east-1:111122223333:workgroup/my_workgroup
    """

    resource_type: str = dataclasses.field(default="workgroup")

    @property
    def workgroup_id(self) -> str:
        return self.resource_id

    @property
    def workgroup_name(self):  # pragma: no cover
        raise AttributeError("You cannot get the human friendly name from the ARN!")


@dataclasses.dataclass
class RedshiftServerlessSnapshot(_RedshiftServerlessCommon):
    """
    Example: arn:aws:redshift-serverless:us-east-1:111122223333:snapshot/my_snapshot
    """

    resource_type: str = dataclasses.field(default="snapshot")

    @property
    def snapshot_id(self) -> str:
        return self.resource_id

    @property
    def snapshot_name(self):  # pragma: no cover
        raise AttributeError("You cannot get the human friendly name from the ARN!")


@dataclasses.dataclass
class RedshiftServerlessManagedVpcEndpoint(_RedshiftServerlessCommon):
    """
    Example: arn:aws:redshift-serverless:us-east-1:111122223333:managedvpcendpoint/my_vpc_endpoint
    """

    resource_type: str = dataclasses.field(default="managedvpcendpoint")



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class Glue(_SlashSeparatedRegional):
    """
    todo: docstring
    """
    service: str = dataclasses.field(default="glue")


@dataclasses.dataclass
class _GlueCommon(Glue):
    """
    todo: docstring
    """

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=name,
        )


@dataclasses.dataclass
class GlueDatabase(Glue):
    """
    Example: arn:aws:glue:us-east-1:111122223333:database/db1
    """
    resource_type: str = dataclasses.field(default="database")

    @property
    def database_name(self) -> str:  # pragma: no cover
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        database_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=database_name,
        )


@dataclasses.dataclass
class GlueTable(Glue):
    """
    Example: arn:aws:glue:us-east-1:111122223333:table/db1/tbl
    """
    resource_type: str = dataclasses.field(default="table")

    @property
    def database_name(self) -> str:  # pragma: no cover
        return self.resource_id.split("/")[0]

    @property
    def table_name(self) -> str:  # pragma: no cover
        return self.resource_id.split("/")[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        database_name: str,
        table_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{database_name}/{table_name}",
        )


@dataclasses.dataclass
class GlueCrawler(_GlueCommon):
    """
    Example: arn:aws:glue:us-east-1:111122223333:crawler/mycrawler
    """
    resource_type: str = dataclasses.field(default="crawler")

    @property
    def crawler_name(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class GlueJob(_GlueCommon):
    """
    Example: arn:aws:glue:us-east-1:111122223333:job/testjob
    """
    resource_type: str = dataclasses.field(default="job")

    @property
    def job_name(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class GlueTrigger(_GlueCommon):
    """
    Example: arn:aws:glue:us-east-1:111122223333:trigger/sampletrigger
    """
    resource_type: str = dataclasses.field(default="trigger")

    @property
    def trigger_name(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class GlueMLTransform(_GlueCommon):
    """
    Example: arn:aws:glue:us-east-1:111122223333:mlTransform/tfm-1234567890
    """
    resource_type: str = dataclasses.field(default="mlTransform")

    @property
    def ml_transform_name(self) -> str:  # pragma: no cover
        return self.resource_id



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class SystemManager(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="ssm")


@dataclasses.dataclass
class SSMParameter(SystemManager):
    """
    Example: arn:aws:ssm:us-east-1:807388292768:parameter/acu_e5f245a1_test
    """
    resource_type: str = dataclasses.field(default="parameter")

    @property
    def parameter_name(self) -> str:
        """
        It is the stack ARN.
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=name,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _ColonSeparatedRegional


@dataclasses.dataclass
class Rds(_ColonSeparatedRegional):
    """
    todo: docstring
    """
    service: str = dataclasses.field(default="rds")


@dataclasses.dataclass
class _RdsCommon(Rds):
    """
    todo: docstring
    """

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        resource_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=resource_id,
        )


@dataclasses.dataclass
class RdsDBInstance(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:db:my-mysql-instance-1
    """
    resource_type: str = dataclasses.field(default="db")

    @property
    def db_instance_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsDBCluster(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:cluster:my-aurora-cluster-1
    """
    resource_type: str = dataclasses.field(default="cluster")

    @property
    def db_cluster_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsEventSubscription(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:es:my-subscription
    """
    resource_type: str = dataclasses.field(default="es")

    @property
    def db_event_subscription_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsDBOptionGroup(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:og:my-og
    """
    resource_type: str = dataclasses.field(default="og")

    @property
    def db_option_group_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsDBParameterGroup(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-2:123456789012:pg:my-param-enable-logs
    """
    resource_type: str = dataclasses.field(default="pg")

    @property
    def db_parameter_group_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsDBClusterParameterGroup(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:cluster-pg:my-cluster-param-timezone
    """
    resource_type: str = dataclasses.field(default="cluster-pg")

    @property
    def db_cluster_parameter_group_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsReservedDBInstance(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:ri:my-reserved-postgresql
    """
    resource_type: str = dataclasses.field(default="ri")

    @property
    def reserved_db_instance_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsDBSecurityGroup(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:secgrp:my-public
    """
    resource_type: str = dataclasses.field(default="secgrp")

    @property
    def db_security_group_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RdsDBInstanceSnapshot(_RdsCommon):
    """
    Example:

    - arn:aws:rds:us-east-1:111122223333:snapshot:rds:my-mysql-db-2020-01-01-00-00
    - arn:aws:rds:us-east-1:111122223333:snapshot:my-mysql-db-snap
    """
    resource_type: str = dataclasses.field(default="snapshot")

    @property
    def db_instance_snapshot_name(self) -> str:  # pragma: no cover
        return self.resource_id

    def is_system_managed(self) -> bool:
        return self.db_instance_snapshot_name.startswith("rds:")


@dataclasses.dataclass
class RdsDBClusterSnapshot(_RdsCommon):
    """
    Example:

    - arn:aws:rds:us-east-1:111122223333:cluster-snapshot:rds:my-aurora-cluster-2020-01-01-00-00
    - arn:aws:rds:us-east-1:111122223333:cluster-snapshot:my-aurora-cluster-snap
    """
    resource_type: str = dataclasses.field(default="cluster-snapshot")

    @property
    def db_cluster_snapshot_name(self) -> str:  # pragma: no cover
        return self.resource_id

    def is_system_managed(self) -> bool:
        return self.db_cluster_snapshot_name.startswith("rds:")


@dataclasses.dataclass
class RdsDBSubnetGroup(_RdsCommon):
    """
    Example: arn:aws:rds:us-east-1:111122223333:subgrp:my-subnet-10
    """
    resource_type: str = dataclasses.field(default="subgrp")

    @property
    def db_subnet_group_name(self) -> str:  # pragma: no cover
        return self.resource_id



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class OpenSearch(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="es")


@dataclasses.dataclass
class OpenSearchServerless(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="aoss")


@dataclasses.dataclass
class OpenSearchDomain(OpenSearch):
    """
    Example: arn:aws:es:us-east-1:111122223333:domain/my_domain
    """
    resource_type: str = dataclasses.field(default="domain")

    @property
    def domain_name(self) -> str:
        """
        "my_domain" part of
        arn:aws:es:us-east-1:111122223333:domain/my_domain
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        domain_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=domain_name,
        )


@dataclasses.dataclass
class OpenSearchServerlessCollection(OpenSearchServerless):
    """
    Example: arn:aws:es:us-east-1:111122223333:domain/my_domain
    """
    resource_type: str = dataclasses.field(default="collection")

    @property
    def collection_id(self) -> str:
        """
        "collection_id" part of
        arn:aws:aoss:us-east-1:111122223333:collection/collection_id
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        collection_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=collection_id,
        )


@dataclasses.dataclass
class OpenSearchServerlessDashboard(OpenSearchServerless):
    """
    Example: arn:aws:aoss:us-east-1:111122223333:dashboards/default
    """
    resource_type: str = dataclasses.field(default="dashboards")

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id="default",
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class A2I(_SlashSeparatedRegional):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:flow-definition/my-flow
    """

    service: str = dataclasses.field(default="sagemaker")

    @property
    def name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=name,
        )


@dataclasses.dataclass
class A2IHumanReviewWorkflow(A2I):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:flow-definition/my_flow
    """

    resource_type: str = dataclasses.field(default="flow-definition")

    @property
    def a2i_human_review_workflow_name(self) -> str:
        return self.resource_id


@dataclasses.dataclass
class A2IHumanLoop(A2I):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:human-loop/1a2b3c
    """

    resource_type: str = dataclasses.field(default="human-loop")

    @property
    def a2i_human_loop_name(self) -> str:
        return self.resource_id


@dataclasses.dataclass
class A2IWorkerTaskTemplate(A2I):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:human-task-ui/my-ui
    """

    resource_type: str = dataclasses.field(default="human-task-ui")

    @property
    def a2i_worker_task_template_name(self) -> str:
        return self.resource_id



# -*- coding: utf-8 -*-


def test():
    from aws_arns import api

    _ = api.Arn
    _ = api.BaseArn
    _ = api.is_arn_instance
    _ = api.parse_arn
    _ = api.AwsPartitionEnum
    _ = api.res.ApiGatewayV1Authorizer
    _ = api.res.ApiGatewayV1Deployment
    _ = api.res.ApiGatewayV1Integration
    _ = api.res.ApiGatewayV1Model
    _ = api.res.ApiGatewayV1Route
    _ = api.res.ApiGatewayV1Stage
    _ = api.res.ApiGatewayV2Authorizer
    _ = api.res.ApiGatewayV2Deployment
    _ = api.res.ApiGatewayV2Integration
    _ = api.res.ApiGatewayV2Model
    _ = api.res.ApiGatewayV2Route
    _ = api.res.ApiGatewayV2Stage
    _ = api.res.LambdaFunction
    _ = api.res.LambdaLayer
    _ = api.res.BatchComputeEnvironment
    _ = api.res.BatchJob
    _ = api.res.BatchJobDefinition
    _ = api.res.BatchJobQueue
    _ = api.res.BatchSchedulingPolicy
    _ = api.res.CloudFormationChangeSet
    _ = api.res.CloudFormationStack
    _ = api.res.CloudFormationStackSet
    _ = api.res.CodeBuildBatchRun
    _ = api.res.CodeBuildProject
    _ = api.res.CodeBuildRun
    _ = api.res.CodeCommitRepository
    _ = api.res.CodePipelinePipeline
    _ = api.res.DynamodbGlobalTable
    _ = api.res.DynamodbTable
    _ = api.res.DynamodbTableBackup
    _ = api.res.DynamodbTableExport
    _ = api.res.DynamodbTableImport
    _ = api.res.DynamodbTableIndex
    _ = api.res.DynamodbTableStream
    _ = api.res.ClientVPNEndpoint
    _ = api.res.DHCPOptionSet
    _ = api.res.EbsSnapshot
    _ = api.res.EbsVolume
    _ = api.res.Ec2Image
    _ = api.res.Ec2Instance
    _ = api.res.Ec2KeyPair
    _ = api.res.Ec2NetworkInterface
    _ = api.res.ElasticIpAllocation
    _ = api.res.InternetGateway
    _ = api.res.NatGateway
    _ = api.res.NetworkACL
    _ = api.res.RouteTable
    _ = api.res.SecurityGroup
    _ = api.res.SecurityGroupRule
    _ = api.res.SiteToSiteVPNConnection
    _ = api.res.Subnet
    _ = api.res.TransitGateway
    _ = api.res.TransitGatewayAttachment
    _ = api.res.Vpc
    _ = api.res.VpcCustomGateway
    _ = api.res.VpcEndpoint
    _ = api.res.VpcPeeringConnection
    _ = api.res.VpcPrivateGateway
    _ = api.res.EcrRepository
    _ = api.res.EcsCluster
    _ = api.res.EcsContainerInstance
    _ = api.res.EcsService
    _ = api.res.EcsTaskDefinition
    _ = api.res.EcsTaskRun
    _ = api.res.GlueCrawler
    _ = api.res.GlueDatabase
    _ = api.res.GlueJob
    _ = api.res.GlueMLTransform
    _ = api.res.GlueTable
    _ = api.res.GlueTrigger
    _ = api.res.IamGroup
    _ = api.res.IamInstanceProfile
    _ = api.res.IamPolicy
    _ = api.res.IamRole
    _ = api.res.IamUser
    _ = api.res.KmsAlias
    _ = api.res.KmsKey
    _ = api.res.CloudWatchLogGroup
    _ = api.res.CloudWatchLogGroupStream
    _ = api.res.OpenSearchDomain
    _ = api.res.OpenSearchServerlessCollection
    _ = api.res.OpenSearchServerlessDashboard
    _ = api.res.RdsDBCluster
    _ = api.res.RdsDBClusterParameterGroup
    _ = api.res.RdsDBClusterSnapshot
    _ = api.res.RdsDBInstance
    _ = api.res.RdsDBInstanceSnapshot
    _ = api.res.RdsDBOptionGroup
    _ = api.res.RdsDBParameterGroup
    _ = api.res.RdsDBSecurityGroup
    _ = api.res.RdsDBSubnetGroup
    _ = api.res.RdsEventSubscription
    _ = api.res.RdsReservedDBInstance
    _ = api.res.RedshiftCluster
    _ = api.res.RedshiftDatabaseName
    _ = api.res.RedshiftDatabaseUserGroup
    _ = api.res.RedshiftParameterGroup
    _ = api.res.RedshiftSecurityGroup
    _ = api.res.RedshiftServerlessManagedVpcEndpoint
    _ = api.res.RedshiftServerlessNamespace
    _ = api.res.RedshiftServerlessSnapshot
    _ = api.res.RedshiftServerlessWorkgroup
    _ = api.res.RedshiftSnapshot
    _ = api.res.RedshiftSnapshotSchedule
    _ = api.res.RedshiftSubnetGroup
    _ = api.res.S3Bucket
    _ = api.res.S3Object
    _ = api.res.SageMakerAction
    _ = api.res.SageMakerAlgorithm
    _ = api.res.SageMakerApp
    _ = api.res.SageMakerAppImageConfig
    _ = api.res.SageMakerAutomlJob
    _ = api.res.SageMakerCodeRepository
    _ = api.res.SageMakerCompilationJob
    _ = api.res.SageMakerContext
    _ = api.res.SageMakerDataQualityJobDefinition
    _ = api.res.SageMakerDevice
    _ = api.res.SageMakerDeviceFleet
    _ = api.res.SageMakerDomain
    _ = api.res.SageMakerEndpoint
    _ = api.res.SageMakerEndpointConfig
    _ = api.res.SageMakerExperiment
    _ = api.res.SageMakerExperimentTrial
    _ = api.res.SageMakerExperimentTrialComponent
    _ = api.res.SageMakerFeatureGroup
    _ = api.res.SageMakerHub
    _ = api.res.SageMakerHubContent
    _ = api.res.SageMakerHyperParameterTuningJob
    _ = api.res.SageMakerImage
    _ = api.res.SageMakerImageVersion
    _ = api.res.SageMakerInferenceExperiment
    _ = api.res.SageMakerInferenceRecommendationsJob
    _ = api.res.SageMakerLabelingJob
    _ = api.res.SageMakerModel
    _ = api.res.SageMakerModelBiasJobDefinition
    _ = api.res.SageMakerModelCard
    _ = api.res.SageMakerModelCardExportJob
    _ = api.res.SageMakerModelExplainabilityJobDefinition
    _ = api.res.SageMakerModelPackage
    _ = api.res.SageMakerModelPackageGroup
    _ = api.res.SageMakerModelQualityJobDefinition
    _ = api.res.SageMakerMonitoringSchedule
    _ = api.res.SageMakerMonitoringScheduleAlert
    _ = api.res.SageMakerNotebookInstance
    _ = api.res.SageMakerPipeline
    _ = api.res.SageMakerPipelineExecution
    _ = api.res.SageMakerProcessingJob
    _ = api.res.SageMakerSharedModel
    _ = api.res.SageMakerSharedModelEvent
    _ = api.res.SageMakerSpace
    _ = api.res.SageMakerStudioLifecycleConfig
    _ = api.res.SageMakerTrainingJob
    _ = api.res.SageMakerTransformJob
    _ = api.res.SageMakerUserProfile
    _ = api.res.SageMakerWorkforce
    _ = api.res.SageMakerWorkteam
    _ = api.res.A2IHumanLoop
    _ = api.res.A2IHumanReviewWorkflow
    _ = api.res.A2IWorkerTaskTemplate
    _ = api.res.SecretManagerSecret
    _ = api.res.SnsSubscription
    _ = api.res.SnsTopic
    _ = api.res.SqsQueue
    _ = api.res.SSMParameter
    _ = api.res.SfnExpressStateMachineExecution
    _ = api.res.SfnStandardStateMachineExecution
    _ = api.res.SfnStateMachine


if __name__ == "__main__":
    from aws_arns.tests.helper import run_cov_test

    run_cov_test(__file__, "aws_arns.api", preview=False)


# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import typing as T
import dataclasses

from ..model import _ColonSeparatedRegional


@dataclasses.dataclass
class StepFunction(_ColonSeparatedRegional):
    service: str = dataclasses.field(default="states")


@dataclasses.dataclass
class SfnStateMachine(StepFunction):
    """
    Example:

    - arn:aws:states:us-east-1:111122223333:stateMachine:standard_test
    - arn:aws:states:us-east-1:807388292768:stateMachine:standard_test:1
    - arn:aws:states:us-east-1:807388292768:stateMachine:standard_test:LIVE
    """

    resource_type: str = dataclasses.field(default="stateMachine")

    @property
    def name(self) -> str:
        return self.resource_id.split(":")[0]

    @property
    def version(self) -> T.Optional[str]:
        words = self.resource_id.split(":", 1)
        if len(words) == 2:
            token = words[1]
            if token.isdigit():
                return token
            else:
                raise ValueError(f"Invalid version: {token}")
        else:
            return None

    @property
    def alias(self) -> T.Optional[str]:
        words = self.resource_id.split(":", 1)
        if len(words) == 2:
            token = words[1]
            if token.isdigit():
                raise ValueError(f"Invalid version: {token}")
            else:
                return token
        else:
            return None

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
        version: T.Optional[T.Union[str, int]] = None,
        alias: T.Optional[str] = None,
    ):
        """
        Factory method.
        """
        if version is not None and alias is not None:  # pragma: no cover
            raise ValueError("Cannot specify both version and alias")

        if version is not None:
            resource_id = f"{name}:{version}"
        elif alias is not None:
            resource_id = f"{name}:{alias}"
        else:
            resource_id = name

        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=resource_id,
        )


@dataclasses.dataclass
class _SfnStateMachineExecutionCommon(StepFunction):
    """
    Example:

    - arn:aws:states:us-east-1:111122223333:execution:standard_test:1d858cf6-613f-4576-b94f-e0d654c23843
    - arn:aws:states:us-east-1:111122223333:express:express_test:e935dec6-e748-4977-a2f2-32eeb83d81da:b2f7726e-9b98-4a49-a6c4-9cf23a61f180
    """

    @property
    def state_machine_name(self) -> str:
        return self.resource_id.split(":", 1)[0]

    @property
    def exec_id(self) -> str:
        return self.resource_id.split(":", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        state_machine_name: str,
        exec_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{state_machine_name}:{exec_id}",
        )

@dataclasses.dataclass
class SfnStandardStateMachineExecution(_SfnStateMachineExecutionCommon):
    """
    Example:

    - arn:aws:states:us-east-1:111122223333:execution:standard_test:1d858cf6-613f-4576-b94f-e0d654c23843
    """
    resource_type: str = dataclasses.field(default="execution")


@dataclasses.dataclass
class SfnExpressStateMachineExecution(_SfnStateMachineExecutionCommon):
    """
    Example:

    - arn:aws:states:us-east-1:111122223333:express:express_test:e935dec6-e748-4977-a2f2-32eeb83d81da:b2f7726e-9b98-4a49-a6c4-9cf23a61f180
    """
    resource_type: str = dataclasses.field(default="express")



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _ColonSeparatedRegional


@dataclasses.dataclass
class SecretManager(_ColonSeparatedRegional):
    service: str = dataclasses.field(default="secretsmanager")


@dataclasses.dataclass
class SecretManagerSecret(SecretManager):
    """
    Example: "arn:aws:secretsmanager:us-east-1:111122223333:secret:MyFolder/MySecret-a1b2c3"
    """
    resource_type: str = dataclasses.field(default="secret")

    @property
    def secret_name(self) -> str:
        """
        "MyFolder/MySecret" part of
        arn:aws:secretsmanager:us-east-1:111122223333:secret:MyFolder/MySecret-a1b2c3
        """
        return "-".join(self.resource_id.split("-")[:-1])

    @property
    def long_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        long_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=long_name,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _ColonSeparatedRegional


@dataclasses.dataclass
class CloudWatchLogs(_ColonSeparatedRegional):
    service: str = dataclasses.field(default="logs")


@dataclasses.dataclass
class CloudWatchLogGroup(CloudWatchLogs):
    """
    Example: arn:aws:logs:us-east-1:111122223333:log-group:/aws/lambda/my-func:*
    """

    resource_type: str = dataclasses.field(default="log-group")

    @property
    def log_group_name(self) -> str:
        """
        The "/aws/lambda/my-func:*" part of
        arn:aws:logs:us-east-1:111122223333:log-group:/aws/lambda/my-func:*
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        log_group_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=log_group_name,
        )


@dataclasses.dataclass
class CloudWatchLogGroupStream(CloudWatchLogs):
    """
    Example: arn:aws:logs:us-east-1:111122223333:log-group:my-log-group*:log-stream:my-log-stream*
    """

    resource_type: str = dataclasses.field(default="log-group")

    @property
    def log_group_name(self) -> str:
        """
        The "my-log-group*" part of
        arn:aws:logs:us-east-1:111122223333:log-group:my-log-group*:log-stream:my-log-stream*
        """
        return self.resource_id.split(":", 2)[0]

    @property
    def stream_name(self) -> str:
        """
        The "my-log-stream*" part of
        arn:aws:logs:us-east-1:111122223333:log-group:my-log-group*:log-stream:my-log-stream*
        """
        return self.resource_id.split(":", 2)[2]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        log_group_name: str,
        stream_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{log_group_name}:log-stream:{stream_name}",
        )



# -*- coding: utf-8 -*-

import pytest

from aws_arns.model import Arn
from aws_arns.tests.test_arns import arns


class TestArn:
    def test_specific(self):
        arn = Arn.from_arn("arn:aws:s3:::my-bucket")
        assert arn.partition == "aws"
        assert arn.service == "s3"
        assert arn.region == None
        assert arn.account_id == None
        assert arn.resource_type == None
        assert arn.resource_id == "my-bucket"
        assert arn.sep == None

        arn.with_cn_partition()
        assert arn.partition == "aws-cn"

        arn.with_us_gov_partition()
        assert arn.partition == "aws-us-gov"

        arn = Arn.from_arn("arn:aws:s3:::my-bucket/file.txt")
        assert arn.partition == "aws"
        assert arn.service == "s3"
        assert arn.region == None
        assert arn.account_id == None
        assert arn.resource_type == None
        assert arn.resource_id == "my-bucket/file.txt"
        assert arn.sep == None

        arn = Arn.from_arn("arn:aws:iam::111122223333:my-role")
        assert arn.partition == "aws"
        assert arn.service == "iam"
        assert arn.region == None
        assert arn.account_id == "111122223333"
        assert arn.resource_type == None
        assert arn.resource_id == "my-role"
        assert arn.sep == None

        arn = Arn.from_arn(
            "arn:aws:sns:us-east-1:111122223333:my_topic:a07e1034-10c0-47a6-83c2-552cfcca42db"
        )
        assert arn.partition == "aws"
        assert arn.service == "sns"
        assert arn.region == "us-east-1"
        assert arn.account_id == "111122223333"
        assert arn.resource_type == None
        assert arn.resource_id == "my_topic:a07e1034-10c0-47a6-83c2-552cfcca42db"
        assert arn.sep == None

        arn = Arn.from_arn("arn:aws:lambda:us-east-1:111122223333:function:my-func")
        assert arn.partition == "aws"
        assert arn.service == "lambda"
        assert arn.region == "us-east-1"
        assert arn.account_id == "111122223333"
        assert arn.resource_type == "function"
        assert arn.resource_id == "my-func"
        assert arn.sep == ":"

        arn = Arn.from_arn(
            "arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/1a2b3c"
        )
        assert arn.partition == "aws"
        assert arn.service == "cloudformation"
        assert arn.region == "us-east-1"
        assert arn.account_id == "111122223333"
        assert arn.resource_type == "stack"
        assert arn.resource_id == "my-stack/1a2b3c"
        assert arn.sep == "/"

        arn = Arn.from_arn(
            "arn:aws:cloudformation:us-east-1:111122223333:stackset/my-stack-set:1a2b3c",
        )
        assert arn.partition == "aws"
        assert arn.service == "cloudformation"
        assert arn.region == "us-east-1"
        assert arn.account_id == "111122223333"
        assert arn.resource_type == "stackset"
        assert arn.resource_id == "my-stack-set:1a2b3c"
        assert arn.sep == "/"

    def test_from_and_to(self):
        for arn_str in arns:
            arn = Arn.from_arn(arn_str)
            assert arn.to_arn() == arn_str

    def test_error(self):
        with pytest.raises(ValueError):
            Arn.from_arn("hello")

        with pytest.raises(ValueError):
            Arn()


if __name__ == "__main__":
    from aws_arns.tests.helper import run_cov_test

    run_cov_test(__file__, "aws_arns.model", preview=False)



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class SageMaker(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="sagemaker")


@dataclasses.dataclass
class _SageMakerCommon(SageMaker):
    @property
    def name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=name,
        )


@dataclasses.dataclass
class SageMakerAction(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:action/my_action
    """

    resource_type: str = dataclasses.field(default="action")


@dataclasses.dataclass
class SageMakerAlgorithm(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:algorithm/my_algorithm
    """

    resource_type: str = dataclasses.field(default="algorithm")


@dataclasses.dataclass
class SageMakerAppImageConfig(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:app-image-config/my_app_image_config
    """

    resource_type: str = dataclasses.field(default="app-image-config")


@dataclasses.dataclass
class SageMakerAutomlJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:automl-job/my_automl_job
    """

    resource_type: str = dataclasses.field(default="automl-job")


@dataclasses.dataclass
class SageMakerCodeRepository(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:code-repository/my_code_repository
    """

    resource_type: str = dataclasses.field(default="code-repository")


@dataclasses.dataclass
class SageMakerCompilationJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:compilation-job/my_compilation_job
    """

    resource_type: str = dataclasses.field(default="compilation-job")


@dataclasses.dataclass
class SageMakerContext(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:context/my_context
    """

    resource_type: str = dataclasses.field(default="context")


@dataclasses.dataclass
class SageMakerDataQualityJobDefinition(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:data-quality-job-definition/my_data_quality_job_definition
    """

    resource_type: str = dataclasses.field(default="data-quality-job-definition")


@dataclasses.dataclass
class SageMakerEndpoint(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:endpoint/my_endpoint
    """

    resource_type: str = dataclasses.field(default="endpoint")


@dataclasses.dataclass
class SageMakerEndpointConfig(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:endpoint-config/my_endpoint_config
    """

    resource_type: str = dataclasses.field(default="endpoint-config")


@dataclasses.dataclass
class SageMakerExperiment(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:experiment/my_experiment
    """

    resource_type: str = dataclasses.field(default="experiment")


@dataclasses.dataclass
class SageMakerExperimentTrial(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:experiment-trial/
    """

    resource_type: str = dataclasses.field(default="experiment-trial")


@dataclasses.dataclass
class SageMakerExperimentTrialComponent(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:experiment-trial-component/
    """

    resource_type: str = dataclasses.field(default="experiment-trial-component")


@dataclasses.dataclass
class SageMakerFeatureGroup(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:feature-group/
    """

    resource_type: str = dataclasses.field(default="feature-group")


@dataclasses.dataclass
class SageMakerHub(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:hub/my_hub
    """

    resource_type: str = dataclasses.field(default="hub")


@dataclasses.dataclass
class SageMakerHubContent(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:hub-content/my_hub/my_hub_content_type/my_hub_content_name
    """

    resource_type: str = dataclasses.field(default="hub-content")


@dataclasses.dataclass
class SageMakerHyperParameterTuningJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:hyper-parameter-tuning-job/my_hyper_parameter_tuning_job
    """

    resource_type: str = dataclasses.field(default="hyper-parameter-tuning-job")


@dataclasses.dataclass
class SageMakerImage(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:image/my_image
    """

    resource_type: str = dataclasses.field(default="image")


@dataclasses.dataclass
class SageMakerImageVersion(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:image-version/my_image/1
    """

    resource_type: str = dataclasses.field(default="image-version")


@dataclasses.dataclass
class SageMakerInferenceExperiment(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:inference-experiment/my_inference_experiment
    """

    resource_type: str = dataclasses.field(default="inference-experiment")


@dataclasses.dataclass
class SageMakerInferenceRecommendationsJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:inference-recommendations-job/my_inference_recommendations_job
    """

    resource_type: str = dataclasses.field(default="inference-recommendations-job")


@dataclasses.dataclass
class SageMakerLabelingJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:labeling-job/my_labeling_job
    """

    resource_type: str = dataclasses.field(default="labeling-job")


@dataclasses.dataclass
class SageMakerModel(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model/my_model
    """

    resource_type: str = dataclasses.field(default="model")


@dataclasses.dataclass
class SageMakerModelBiasJobDefinition(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-bias-job-definition/my_model_bias_job_definition
    """

    resource_type: str = dataclasses.field(default="model-bias-job-definition")


@dataclasses.dataclass
class SageMakerModelCard(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-card/my_model_card
    """

    resource_type: str = dataclasses.field(default="model-card")


@dataclasses.dataclass
class SageMakerModelCardExportJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-card/my_model_card/export-job/my_export_job_name
    """

    resource_type: str = dataclasses.field(default="model-card")


@dataclasses.dataclass
class SageMakerModelExplainabilityJobDefinition(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-explainability-job-definition/my_model_explainability_job_definition
    """

    resource_type: str = dataclasses.field(
        default="model-explainability-job-definition"
    )


@dataclasses.dataclass
class SageMakerModelPackage(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-package/my_model_package
    """

    resource_type: str = dataclasses.field(default="model-package")


@dataclasses.dataclass
class SageMakerModelPackageGroup(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-package-group/my_model_package_group
    """

    resource_type: str = dataclasses.field(default="model-package-group")


@dataclasses.dataclass
class SageMakerModelQualityJobDefinition(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:model-quality-job-definition/my_model_quality_job_definition
    """

    resource_type: str = dataclasses.field(default="model-quality-job-definition")


@dataclasses.dataclass
class SageMakerMonitoringSchedule(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:monitoring-schedule/my_monitoring_schedule
    """

    resource_type: str = dataclasses.field(default="monitoring-schedule")


@dataclasses.dataclass
class SageMakerMonitoringScheduleAlert(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:monitoring-schedule/my_monitoring_schedule/alert/my_monitoring_schedule_alert_name
    """

    resource_type: str = dataclasses.field(default="monitoring-schedule")


@dataclasses.dataclass
class SageMakerNotebookInstance(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:notebook-instance/my_notebook_instance
    """

    resource_type: str = dataclasses.field(default="notebook-instance")


@dataclasses.dataclass
class SageMakerPipeline(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:pipeline/my_pipeline
    """

    resource_type: str = dataclasses.field(default="pipeline")


@dataclasses.dataclass
class SageMakerPipelineExecution(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:pipeline/my_pipeline/execution/1a2b3c
    """

    resource_type: str = dataclasses.field(default="pipeline")


@dataclasses.dataclass
class SageMakerProcessingJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:processing-job/my_processing_job
    """

    resource_type: str = dataclasses.field(default="processing-job")


@dataclasses.dataclass
class SageMakerSharedModel(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:shared-model/share_model_id
    """

    resource_type: str = dataclasses.field(default="shared-model")


@dataclasses.dataclass
class SageMakerSharedModelEvent(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:shared-model-event/share_model_event_id
    """

    resource_type: str = dataclasses.field(default="shared-model-event")


@dataclasses.dataclass
class SageMakerTrainingJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:training-job/my_training_job
    """

    resource_type: str = dataclasses.field(default="training-job")


@dataclasses.dataclass
class SageMakerTransformJob(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:transform-job/my_transform_job
    """

    resource_type: str = dataclasses.field(default="transform-job")


@dataclasses.dataclass
class SageMakerWorkforce(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:workforce/my_workforce
    """

    resource_type: str = dataclasses.field(default="workforce")


@dataclasses.dataclass
class SageMakerWorkteam(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:workteam/my_workteam
    """

    resource_type: str = dataclasses.field(default="workteam")


@dataclasses.dataclass
class SageMakerDomain(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:domain/domain_id
    """

    resource_type: str = dataclasses.field(default="domain")


@dataclasses.dataclass
class SageMakerUserProfile(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:user-profile/domain_id/my_user_profile
    """

    resource_type: str = dataclasses.field(default="user-profile")


@dataclasses.dataclass
class SageMakerSpace(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:space/domain_id/my_space
    """

    resource_type: str = dataclasses.field(default="space")


@dataclasses.dataclass
class SageMakerApp(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:app/domain_id/my_user_profile/my_app_type/my_app_name
    """

    resource_type: str = dataclasses.field(default="app")


@dataclasses.dataclass
class SageMakerStudioLifecycleConfig(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:studio-lifecycle-config/my_studio_lifecycle_config
    """

    resource_type: str = dataclasses.field(default="studio-lifecycle-config")


@dataclasses.dataclass
class SageMakerDevice(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:device-fleet/fleet_name/device/device_name
    """

    resource_type: str = dataclasses.field(default="device-fleet")


@dataclasses.dataclass
class SageMakerDeviceFleet(_SageMakerCommon):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:device-fleet/fleet_name
    """

    resource_type: str = dataclasses.field(default="device-fleet")



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class Ec2(_SlashSeparatedRegional):
    """
    todo: docstring
    """

    service: str = dataclasses.field(default="ec2")


@dataclasses.dataclass
class _Ec2Common(Ec2):
    """
    todo: docstring
    """

    _id_prefix: str = None

    @property
    def id_prefix(self) -> str:
        """
        "vpc" part of the "vpc-1234567890abcdef0".
        """
        return "-".join(self.resource_id.split("-")[:-1])

    @property
    def short_id(self) -> str:
        """
        "1234567890abcdef0" part of the "vpc-1234567890abcdef0".
        """
        return self.resource_id.split("-")[-1]

    @property
    def long_id(self) -> str:
        """
        The "vpc-1234567890abcdef0".
        """
        return self.resource_id


@dataclasses.dataclass
class _Ec2CommonRegional(_Ec2Common):
    """
    todo: docstring
    """

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        resource_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=resource_id,
        )


@dataclasses.dataclass
class Ec2Instance(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:instance/i-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="instance")

    _id_prefix = "i"

    @property
    def instance_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class Ec2KeyPair(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:key-pair/key-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="key-pair")

    _id_prefix = "key"

    @property
    def key_name(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class EbsVolume(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:volume/vol-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="volume")

    _id_prefix = "vol"

    @property
    def volume_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class EbsSnapshot(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:snapshot/snap-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="snapshot")

    _id_prefix = "snap"

    @property
    def volume_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class Ec2NetworkInterface(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:network-interface/eni-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="network-interface")

    _id_prefix = "eni"

    @property
    def network_interface_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class Vpc(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:vpc/vpc-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="vpc")

    _id_prefix = "vpc"

    @property
    def instance_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class Subnet(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:subnet/subnet-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="subnet")

    _id_prefix = "subnet"

    @property
    def subnet_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class RouteTable(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:route-table/rtb-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="route-table")

    _id_prefix = "rtb"

    @property
    def route_table_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class InternetGateway(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:internet-gateway/igw-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="internet-gateway")

    _id_prefix = "igw"

    @property
    def internet_gateway_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class NatGateway(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:natgateway/nat-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="natgateway")

    _id_prefix = "nat"

    @property
    def nat_gateway_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class DHCPOptionSet(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:dhcp-options/dopt-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="dhcp-options")

    _id_prefix = "dopt"

    @property
    def dhcp_option_set_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class VpcPeeringConnection(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:vpc-peering-connection/pcx-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="vpc-peering-connection")

    _id_prefix = "pcx"

    @property
    def vpc_peering_connection_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class NetworkACL(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:network-acl/acl-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="network-acl")

    _id_prefix = "acl"

    @property
    def network_acl_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class SecurityGroup(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:security-group/sg-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="security-group")

    _id_prefix = "sg"

    @property
    def sg_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class SecurityGroupRule(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:security-group-rule/sgr-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="security-group-rule")

    _id_prefix = "sgr"

    @property
    def sg_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class VpcEndpoint(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:vpc-endpoint/vpce-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="vpc-endpoint")

    _id_prefix = "vpce"

    @property
    def vpc_endpoint_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class ElasticIpAllocation(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:ipv4pool-ec2/eipalloc-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="ipv4pool-ec2")

    _id_prefix = "eipalloc"

    @property
    def elastic_ip_allocation_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class VpcCustomGateway(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:customer-gateway/cgw-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="customer-gateway")

    _id_prefix = "cgw"

    @property
    def vpc_custom_gateway_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class VpcPrivateGateway(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:vpn-gateway/vgw-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="vpn-gateway")

    _id_prefix = "vgw"

    @property
    def vpc_private_gateway_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class SiteToSiteVPNConnection(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:vpn-connection/vpn-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="vpn-connection")

    _id_prefix = "vpn"

    @property
    def vpn_connection_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class ClientVPNEndpoint(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:client-vpn-endpoint/cvpn-endpoint-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="client-vpn-endpoint")

    _id_prefix = "cvpn-endpoint"

    @property
    def client_vpn_endpoint_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class TransitGateway(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:transit-gateway/tgw-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="transit-gateway")

    _id_prefix = "tgw"

    @property
    def transit_gateway_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class TransitGatewayAttachment(_Ec2CommonRegional):
    """
    Example: arn:aws:ec2:us-east-1:123456789012:transit-gateway-attachment/tgw-attach-1234567890abcdef0
    """

    resource_type: str = dataclasses.field(default="transit-gateway-attachment")

    _id_prefix = "tgw-attach"

    @property
    def transit_gateway_attachment_id(self) -> str:  # pragma: no cover
        return self.resource_id


@dataclasses.dataclass
class Ec2Image(_Ec2Common):
    """
    Example: arn:aws:ec2:us-east-1::image/ami-1234567890abcdef0
    """

    account_id: str = dataclasses.field(default=None)
    resource_type: str = dataclasses.field(default="image")

    _id_prefix = "ami"

    @property
    def ami_id(self) -> str:  # pragma: no cover
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_region: str,
        resource_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            region=aws_region,
            resource_id=resource_id,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _ResourceIdOnlyRegional


@dataclasses.dataclass
class Sqs(_ResourceIdOnlyRegional):
    service: str = dataclasses.field(default="sqs")


@dataclasses.dataclass
class SqsQueue(Sqs):
    """
    Example: arn:aws:sqs:us-east-1:111122223333:my_queue
    """

    @property
    def queue_name(self) -> str:
        """
        Queue short name, if it is fifo queue, it has to end with .fifo.
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        queue_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=queue_name,
        )

    @property
    def queue_url(self) -> str:
        """
        Example: https://sqs.us-east-1.amazonaws.com/111122223333/my_queue
        """
        return (
            f"https://sqs.{self.aws_region}.amazonaws.com"
            f"/{self.aws_account_id}/{self.queue_name}"
        )

    @classmethod
    def from_queue_url(cls, url: str):
        """
        Parse a queue URL into an SQS object.
        """
        parts = url.split("/")
        aws_region = parts[2].split(".")[1]
        aws_account_id = parts[3]
        queue_name = parts[4]
        return cls.new(
            aws_account_id=aws_account_id,
            aws_region=aws_region,
            queue_name=queue_name,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class CodeBuild(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="codebuild")


@dataclasses.dataclass
class CodeBuildProject(CodeBuild):
    """
    Example: arn:aws:codebuild:us-east-1:111122223333:project/my-project
    """

    resource_type: str = dataclasses.field(default="project")

    @property
    def codebuild_project_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=name,
        )


@dataclasses.dataclass
class _CodeBuildRunCommon(CodeBuild):
    """
    arn:aws:codebuild:us-east-1:111122223333:build/my-project:a1b2c3d4
    arn:aws:codebuild:us-east-1:111122223333:build-batch/my-project:a1b2c3d4
    """
    @property
    def codebuild_run_fullname(self) -> str:
        """
        The "my-project:a1b2c3d4" part of:

        - arn:aws:codebuild:us-east-1:111122223333:build/my-project:a1b2c3d4
        - arn:aws:codebuild:us-east-1:111122223333:build-batch/my-project:a1b2c3d4
        """
        return self.resource_id

    @property
    def codebuild_project_name(self) -> str:
        """
        The "my-project" part of:

        - arn:aws:codebuild:us-east-1:111122223333:build/my-project:a1b2c3d4
        - arn:aws:codebuild:us-east-1:111122223333:build-batch/my-project:a1b2c3d4
        """
        return self.resource_id.split(":")[0]

    @property
    def codebuild_run_id(self) -> str:
        """
        The "a1b2c3d4" part of:

        - arn:aws:codebuild:us-east-1:111122223333:build/my-project:a1b2c3d4
        - arn:aws:codebuild:us-east-1:111122223333:build-batch/my-project:a1b2c3d4
        """
        return self.resource_id.split(":")[-1]

    def is_batch_build(self) -> bool:
        return self.resource_type == "build-batch"

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=fullname,
        )


@dataclasses.dataclass
class CodeBuildRun(_CodeBuildRunCommon):
    """
    Example: arn:aws:codebuild:us-east-1:111122223333:build/my-project:a1b2c3d4
    """

    resource_type: str = dataclasses.field(default="build")


@dataclasses.dataclass
class CodeBuildBatchRun(_CodeBuildRunCommon):
    """
    Example: arn:aws:codebuild:us-east-1:111122223333:build-batch/my-project:a1b2c3d4
    """

    resource_type: str = dataclasses.field(default="build-batch")



# -*- coding: utf-8 -*-

from aws_arns.tests.test_arns import arns
from aws_arns.model import is_arn_instance
from aws_arns.parse_arn import parse_arn

def test():
    for arn in arns:
        obj = parse_arn(arn)
        assert is_arn_instance(obj) is True


if __name__ == "__main__":
    from aws_arns.tests.helper import run_cov_test

    run_cov_test(__file__, "aws_arns.parse_arn", preview=False)


# -*- coding: utf-8 -*-

import dataclasses
from . import resource
from .model import Arn


class ParseArn:
    def __init__(self):
        self._mapper = None

    def _load(self):
        _mapper = dict()
        for k, v in resource.__dict__.items():
            try:
                fields = {field.name: field for field in dataclasses.fields(v)}
                service = fields["service"].default
                resource_type = fields["resource_type"].default
                key = f"{service}.{resource_type}"
                if "Nothing" in key:  # pragma: no cover
                    raise ValueError
                _mapper[key] = v
            except TypeError:
                pass
        self._mapper = _mapper

    def __call__(self, arn: str):
        if self._mapper is None:
            self._load()
        obj = Arn.from_arn(arn)
        key = f"{obj.service}.{obj.resource_type}"
        if key in self._mapper:
            return self._mapper[key].from_arn(arn)
        else:
            return obj


parse_arn = ParseArn()



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class Ecs(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="ecs")


@dataclasses.dataclass
class EcsCluster(Ecs):
    """
    Example: arn:aws:ecs:us-east-1:123456789012:cluster/my-cluster-1
    """

    resource_type: str = dataclasses.field(default="cluster")

    @property
    def cluster_name(self) -> str:  # pragma: no cover
        """
        The "my-cluster-1" part of
        arn:aws:ecs:us-east-1:123456789012:cluster/my-cluster-1
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        cluster_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=cluster_name,
        )


@dataclasses.dataclass
class EcsTaskDefinition(Ecs):
    """
    Example: arn:aws:ecs:us-east-1:111122223333:task-definition/my-task:1
    """

    resource_type: str = dataclasses.field(default="task-definition")

    @property
    def task_name(self) -> str:  # pragma: no cover
        """
        The "my-task" part of
        arn:aws:ecs:us-east-1:111122223333:task-definition/my-task:1
        """
        return self.resource_id.split(":", 1)[0]

    @property
    def version(self) -> int:  # pragma: no cover
        """
        The "1" part of
        arn:aws:ecs:us-east-1:111122223333:task-definition/my-task:1
        """
        return int(self.resource_id.split(":", 1)[1])

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        task_name: str,
        version: int,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{task_name}:{version}",
        )


@dataclasses.dataclass
class EcsContainerInstance(Ecs):
    """
    Example: arn:aws:ecs:us-east-1:111122223333:container-instance/my-cluster/container_instance_UUID
    """

    resource_type: str = dataclasses.field(default="container-instance")

    @property
    def cluster_name(self) -> str:  # pragma: no cover
        """
        The "my-cluster" part of
        arn:aws:ecs:us-east-1:111122223333:container-instance/my-cluster/container_instance_UUID
        """
        return self.resource_id.split("/", 1)[0]

    @property
    def container_instance_id(self) -> str:  # pragma: no cover
        """
        The "container_instance_UUID" part of
        arn:aws:ecs:us-east-1:111122223333:container-instance/my-cluster/container_instance_UUID
        """
        return self.resource_id.split("/", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        cluster_name: str,
        container_instance_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{cluster_name}/{container_instance_id}",
        )


@dataclasses.dataclass
class EcsService(Ecs):
    """
    Example: arn:aws:ecs:us-east-1:111122223333:service/service_name
    """

    resource_type: str = dataclasses.field(default="service")

    @property
    def service_name(self) -> str:  # pragma: no cover
        """
        The "service_name" part of
        arn:aws:ecs:us-east-1:111122223333:service/service_name
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        service_name: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=service_name,
        )


@dataclasses.dataclass
class EcsTaskRun(Ecs):
    """
    Example: arn:aws:ecs:us-east-1:123456789012:task/a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE
    """

    resource_type: str = dataclasses.field(default="task")

    @property
    def run_id(self) -> str:  # pragma: no cover
        """
        The "my_cluster/a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE" part of
        arn:aws:ecs:us-east-1:123456789012:task/my_cluster/a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE
        """
        return self.resource_id

    @property
    def cluster_name(self) -> str:  # pragma: no cover
        """
        The "my_cluster" part of
        arn:aws:ecs:us-east-1:123456789012:task/my_cluster/a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE
        """
        return self.resource_id.split("/", 1)[0]

    @property
    def short_id(self) -> str:  # pragma: no cover
        """
        The "a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE" part of
        arn:aws:ecs:us-east-1:123456789012:task/my_cluster/a1b2c3d4-5678-90ab-ccdef-11111EXAMPLE
        """
        return self.resource_id.split("/", 1)[1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        run_id: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=run_id,
        )



# -*- coding: utf-8 -*-

from .srv.apigateway import ApiGatewayV1Authorizer
from .srv.apigateway import ApiGatewayV1Deployment
from .srv.apigateway import ApiGatewayV1Integration
from .srv.apigateway import ApiGatewayV1Model
from .srv.apigateway import ApiGatewayV1Route
from .srv.apigateway import ApiGatewayV1Stage
from .srv.apigateway import ApiGatewayV2Authorizer
from .srv.apigateway import ApiGatewayV2Deployment
from .srv.apigateway import ApiGatewayV2Integration
from .srv.apigateway import ApiGatewayV2Model
from .srv.apigateway import ApiGatewayV2Route
from .srv.apigateway import ApiGatewayV2Stage
from .srv.awslambda import LambdaFunction
from .srv.awslambda import LambdaLayer
from .srv.batch import BatchComputeEnvironment
from .srv.batch import BatchJob
from .srv.batch import BatchJobDefinition
from .srv.batch import BatchJobQueue
from .srv.batch import BatchSchedulingPolicy
from .srv.cloudformation import CloudFormationChangeSet
from .srv.cloudformation import CloudFormationStack
from .srv.cloudformation import CloudFormationStackSet
from .srv.codebuild import CodeBuildBatchRun
from .srv.codebuild import CodeBuildProject
from .srv.codebuild import CodeBuildRun
from .srv.codecommit import CodeCommitRepository
from .srv.codepipeline import CodePipelinePipeline
from .srv.dynamodb import DynamodbGlobalTable
from .srv.dynamodb import DynamodbTable
from .srv.dynamodb import DynamodbTableBackup
from .srv.dynamodb import DynamodbTableExport
from .srv.dynamodb import DynamodbTableImport
from .srv.dynamodb import DynamodbTableIndex
from .srv.dynamodb import DynamodbTableStream
from .srv.ec2 import ClientVPNEndpoint
from .srv.ec2 import DHCPOptionSet
from .srv.ec2 import EbsSnapshot
from .srv.ec2 import EbsVolume
from .srv.ec2 import Ec2Image
from .srv.ec2 import Ec2Instance
from .srv.ec2 import Ec2KeyPair
from .srv.ec2 import Ec2NetworkInterface
from .srv.ec2 import ElasticIpAllocation
from .srv.ec2 import InternetGateway
from .srv.ec2 import NatGateway
from .srv.ec2 import NetworkACL
from .srv.ec2 import RouteTable
from .srv.ec2 import SecurityGroup
from .srv.ec2 import SecurityGroupRule
from .srv.ec2 import SiteToSiteVPNConnection
from .srv.ec2 import Subnet
from .srv.ec2 import TransitGateway
from .srv.ec2 import TransitGatewayAttachment
from .srv.ec2 import Vpc
from .srv.ec2 import VpcCustomGateway
from .srv.ec2 import VpcEndpoint
from .srv.ec2 import VpcPeeringConnection
from .srv.ec2 import VpcPrivateGateway
from .srv.ecr import EcrRepository
from .srv.ecs import EcsCluster
from .srv.ecs import EcsContainerInstance
from .srv.ecs import EcsService
from .srv.ecs import EcsTaskDefinition
from .srv.ecs import EcsTaskRun
from .srv.glue import GlueCrawler
from .srv.glue import GlueDatabase
from .srv.glue import GlueJob
from .srv.glue import GlueMLTransform
from .srv.glue import GlueTable
from .srv.glue import GlueTrigger
from .srv.iam import IamGroup
from .srv.iam import IamInstanceProfile
from .srv.iam import IamPolicy
from .srv.iam import IamRole
from .srv.iam import IamUser
from .srv.kms import KmsAlias
from .srv.kms import KmsKey
from .srv.logs import CloudWatchLogGroup
from .srv.logs import CloudWatchLogGroupStream
from .srv.opensearch import OpenSearchDomain
from .srv.opensearch import OpenSearchServerlessCollection
from .srv.opensearch import OpenSearchServerlessDashboard
from .srv.rds import RdsDBCluster
from .srv.rds import RdsDBClusterParameterGroup
from .srv.rds import RdsDBClusterSnapshot
from .srv.rds import RdsDBInstance
from .srv.rds import RdsDBInstanceSnapshot
from .srv.rds import RdsDBOptionGroup
from .srv.rds import RdsDBParameterGroup
from .srv.rds import RdsDBSecurityGroup
from .srv.rds import RdsDBSubnetGroup
from .srv.rds import RdsEventSubscription
from .srv.rds import RdsReservedDBInstance
from .srv.redshift import RedshiftCluster
from .srv.redshift import RedshiftDatabaseName
from .srv.redshift import RedshiftDatabaseUserGroup
from .srv.redshift import RedshiftParameterGroup
from .srv.redshift import RedshiftSecurityGroup
from .srv.redshift import RedshiftServerlessManagedVpcEndpoint
from .srv.redshift import RedshiftServerlessNamespace
from .srv.redshift import RedshiftServerlessSnapshot
from .srv.redshift import RedshiftServerlessWorkgroup
from .srv.redshift import RedshiftSnapshot
from .srv.redshift import RedshiftSnapshotSchedule
from .srv.redshift import RedshiftSubnetGroup
from .srv.s3 import S3Bucket
from .srv.s3 import S3Object
from .srv.sagemaker import SageMakerAction
from .srv.sagemaker import SageMakerAlgorithm
from .srv.sagemaker import SageMakerApp
from .srv.sagemaker import SageMakerAppImageConfig
from .srv.sagemaker import SageMakerAutomlJob
from .srv.sagemaker import SageMakerCodeRepository
from .srv.sagemaker import SageMakerCompilationJob
from .srv.sagemaker import SageMakerContext
from .srv.sagemaker import SageMakerDataQualityJobDefinition
from .srv.sagemaker import SageMakerDevice
from .srv.sagemaker import SageMakerDeviceFleet
from .srv.sagemaker import SageMakerDomain
from .srv.sagemaker import SageMakerEndpoint
from .srv.sagemaker import SageMakerEndpointConfig
from .srv.sagemaker import SageMakerExperiment
from .srv.sagemaker import SageMakerExperimentTrial
from .srv.sagemaker import SageMakerExperimentTrialComponent
from .srv.sagemaker import SageMakerFeatureGroup
from .srv.sagemaker import SageMakerHub
from .srv.sagemaker import SageMakerHubContent
from .srv.sagemaker import SageMakerHyperParameterTuningJob
from .srv.sagemaker import SageMakerImage
from .srv.sagemaker import SageMakerImageVersion
from .srv.sagemaker import SageMakerInferenceExperiment
from .srv.sagemaker import SageMakerInferenceRecommendationsJob
from .srv.sagemaker import SageMakerLabelingJob
from .srv.sagemaker import SageMakerModel
from .srv.sagemaker import SageMakerModelBiasJobDefinition
from .srv.sagemaker import SageMakerModelCard
from .srv.sagemaker import SageMakerModelCardExportJob
from .srv.sagemaker import SageMakerModelExplainabilityJobDefinition
from .srv.sagemaker import SageMakerModelPackage
from .srv.sagemaker import SageMakerModelPackageGroup
from .srv.sagemaker import SageMakerModelQualityJobDefinition
from .srv.sagemaker import SageMakerMonitoringSchedule
from .srv.sagemaker import SageMakerMonitoringScheduleAlert
from .srv.sagemaker import SageMakerNotebookInstance
from .srv.sagemaker import SageMakerPipeline
from .srv.sagemaker import SageMakerPipelineExecution
from .srv.sagemaker import SageMakerProcessingJob
from .srv.sagemaker import SageMakerSharedModel
from .srv.sagemaker import SageMakerSharedModelEvent
from .srv.sagemaker import SageMakerSpace
from .srv.sagemaker import SageMakerStudioLifecycleConfig
from .srv.sagemaker import SageMakerTrainingJob
from .srv.sagemaker import SageMakerTransformJob
from .srv.sagemaker import SageMakerUserProfile
from .srv.sagemaker import SageMakerWorkforce
from .srv.sagemaker import SageMakerWorkteam
from .srv.sagemaker_a2i import A2IHumanLoop
from .srv.sagemaker_a2i import A2IHumanReviewWorkflow
from .srv.sagemaker_a2i import A2IWorkerTaskTemplate
from .srv.secretmanager import SecretManagerSecret
from .srv.sns import SnsSubscription
from .srv.sns import SnsTopic
from .srv.sqs import SqsQueue
from .srv.ssm import SSMParameter
from .srv.stepfunction import SfnExpressStateMachineExecution
from .srv.stepfunction import SfnStandardStateMachineExecution
from .srv.stepfunction import SfnStateMachine


# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses

from ..model import _SlashSeparatedRegional


@dataclasses.dataclass
class CloudFormation(_SlashSeparatedRegional):
    service: str = dataclasses.field(default="cloudformation")


@dataclasses.dataclass
class CloudFormationStack(CloudFormation):
    """
    Example: arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/a1b2c3d4
    """
    resource_type: str = dataclasses.field(default="stack")

    @property
    def stack_name(self) -> str:
        """
        The "my-stack" part of
        arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/a1b2c3d4
        """
        return self.resource_id.split("/")[0]

    @property
    def short_id(self) -> str:
        """
        The "a1b2c3d4" part of
        arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/a1b2c3d4
        """
        return self.resource_id.split("/")[1]

    @property
    def stack_fullname(self) -> str:
        """
        The "my-stack/a1b2c3d4" part of
        arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/a1b2c3d4
        """
        return self.resource_id

    @property
    def stack_id(self) -> str:
        """
        It is the stack ARN.
        """
        return self.to_arn()

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        stack_name: str,
        short_id: str
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=f"{stack_name}/{short_id}",
        )


@dataclasses.dataclass
class CloudFormationChangeSet(CloudFormation):
    """
    Example: arn:aws:cloudformation:us-east-1:111122223333:changeSet/my-change-set/a1b2c3d4
    """
    resource_type: str = dataclasses.field(default="changeSet")

    @property
    def changeset_fullname(self) -> str:
        """
        The "my-change-set/a1b2c3d4" part of
        arn:aws:cloudformation:us-east-1:111122223333:changeSet/my-change-set/a1b2c3d4
        """
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=fullname,
        )


@dataclasses.dataclass
class CloudFormationStackSet(CloudFormation):
    """
    Example: arn:aws:cloudformation:us-east-1:111122223333:stackset/my-stackset:a1b2c3d4
    """
    resource_type: str = dataclasses.field(default="stackset")

    @property
    def stackset_name(self) -> str:
        return self.resource_id.split(":")[0]

    @property
    def stackset_id(self) -> str:
        return self.resource_id.split(":")[1]

    @property
    def stackset_fullname(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        """
        Factory method.
        """
        return cls(
            account_id=aws_account_id,
            region=aws_region,
            resource_id=fullname,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import dataclasses
from ..model import _Global


@dataclasses.dataclass
class Iam(_Global):
    service: str = dataclasses.field(default="iam")
    sep: str = dataclasses.field(default="/")

    @property
    def short_name(self) -> str:
        return self.resource_id.split("/")[-1]


@dataclasses.dataclass
class IamGroup(Iam):
    """
    Example: arn:aws:iam::111122223333:group/Admin
    """

    resource_type: str = dataclasses.field(default="group")

    @property
    def iam_group_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ):
        return cls(
            account_id=aws_account_id,
            resource_id=name,
        )


@dataclasses.dataclass
class IamUser(Iam):
    """
    Example: arn:aws:iam::111122223333:user/alice
    """

    resource_type: str = dataclasses.field(default="user")

    @property
    def iam_user_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamUser":
        return cls(
            account_id=aws_account_id,
            resource_id=name,
        )


@dataclasses.dataclass
class IamRole(Iam):
    """
    Example: arn:aws:iam::111122223333:role/aws-service-role/batch.amazonaws.com/AWSServiceRoleForBatch
    """

    resource_type: str = dataclasses.field(default="role")

    @property
    def iam_role_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamRole":
        return cls(
            account_id=aws_account_id,
            resource_id=name,
        )

    def is_service_role(self) -> bool:
        return self.iam_role_name.startswith("aws-service-role")


@dataclasses.dataclass
class IamPolicy(Iam):
    """
    Example: arn:aws:iam::111122223333:policy/service-role/codebuild-policy
    """

    resource_type: str = dataclasses.field(default="policy")

    @property
    def iam_policy_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamPolicy":
        return cls(
            account_id=aws_account_id,
            resource_id=name,
        )


@dataclasses.dataclass
class IamInstanceProfile(Iam):
    """
    Example: arn:aws:iam::111122223333:instance-profile/cloud9/AWSCloud9SSMInstanceProfile
    """

    resource_type: str = dataclasses.field(default="instance-profile")

    @property
    def iam_instance_profile_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamInstanceProfile":
        return cls(
            account_id=aws_account_id,
            resource_id=name,
        )



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import typing as T
import dataclasses

from ..model import _ResourceIdOnlyRegional


@dataclasses.dataclass
class ApiGateway(_ResourceIdOnlyRegional):
    account_id: T.Optional[str] = dataclasses.field(default=None)
    service: str = dataclasses.field(default="apigateway")


_api_version_to_name_mapper = {
    1: "restapis",
    2: "apis",
}

_api_name_to_version_mapper = {
    "restapis": 1,
    "apis": 2,
}


@dataclasses.dataclass
class _ApiCommon(ApiGateway):
    """
    todo: docstring
    """

    api_gateway_version: int = dataclasses.field(default=None)
    api_id: str = dataclasses.field(default=None)
    api_res_type: str = dataclasses.field(default=None)
    api_res_path: T.Optional[str] = dataclasses.field(default=None)

    def __post_init__(self):
        super().__post_init__()
        if self.resource_id is None:
            self.resource_id = (
                f"/{_api_version_to_name_mapper[self.api_gateway_version]}"
                f"/{self.api_id}/{self.api_res_type}{self.api_res_path}"
            )
        else:
            parts = self.resource_id.split("/", 4)
            self.api_gateway_version = _api_name_to_version_mapper[parts[1]]
            self.api_id = parts[2]
            self.api_res_type = parts[3]
            if len(parts) == 5:
                self.api_res_path = "/" + parts[4]
            else:  # pragma: no cover
                self.api_res_path = None

    @classmethod
    def new(
        cls,
        aws_region: str,
        api_id: str,
        api_res_path: T.Optional[str] = None,
    ):
        """
        Factory method.
        """
        if api_res_path is None:  # pragma: no cover
            api_res_path = ""
        return cls(
            region=aws_region,
            resource_id=None,
            api_id=api_id,
            api_res_path=api_res_path,
        )


@dataclasses.dataclass
class _ApiGatewayStage(_ApiCommon):
    api_res_type: str = dataclasses.field(default="stages")


@dataclasses.dataclass
class _ApiGatewayDeployment(_ApiCommon):
    api_res_type: str = dataclasses.field(default="deployments")


@dataclasses.dataclass
class _ApiGatewayAuthorizer(_ApiCommon):
    api_res_type: str = dataclasses.field(default="authorizers")


@dataclasses.dataclass
class _ApiGatewayModel(_ApiCommon):
    api_res_type: str = dataclasses.field(default="models")


@dataclasses.dataclass
class _ApiGatewayRoute(_ApiCommon):
    api_res_type: str = dataclasses.field(default="routes")


@dataclasses.dataclass
class _ApiGatewayIntegration(_ApiCommon):
    api_res_type: str = dataclasses.field(default="integrations")


@dataclasses.dataclass
class ApiGatewayV1Stage(_ApiGatewayStage):
    api_gateway_version: int = dataclasses.field(default=1)


@dataclasses.dataclass
class ApiGatewayV1Deployment(_ApiGatewayDeployment):
    api_gateway_version: int = dataclasses.field(default=1)


@dataclasses.dataclass
class ApiGatewayV1Authorizer(_ApiGatewayAuthorizer):
    api_gateway_version: int = dataclasses.field(default=1)


@dataclasses.dataclass
class ApiGatewayV1Model(_ApiGatewayModel):
    api_gateway_version: int = dataclasses.field(default=1)


@dataclasses.dataclass
class ApiGatewayV1Route(_ApiGatewayRoute):
    api_gateway_version: int = dataclasses.field(default=1)


@dataclasses.dataclass
class ApiGatewayV1Integration(_ApiGatewayIntegration):
    api_gateway_version: int = dataclasses.field(default=1)


@dataclasses.dataclass
class ApiGatewayV2Stage(_ApiGatewayStage):
    api_gateway_version: int = dataclasses.field(default=2)


@dataclasses.dataclass
class ApiGatewayV2Deployment(_ApiGatewayDeployment):
    api_gateway_version: int = dataclasses.field(default=2)


@dataclasses.dataclass
class ApiGatewayV2Authorizer(_ApiGatewayAuthorizer):
    api_gateway_version: int = dataclasses.field(default=2)


@dataclasses.dataclass
class ApiGatewayV2Model(_ApiGatewayModel):
    api_gateway_version: int = dataclasses.field(default=2)


@dataclasses.dataclass
class ApiGatewayV2Route(_ApiGatewayRoute):
    api_gateway_version: int = dataclasses.field(default=2)


@dataclasses.dataclass
class ApiGatewayV2Integration(_ApiGatewayIntegration):
    api_gateway_version: int = dataclasses.field(default=2)



# -*- coding: utf-8 -*-

import typing as T
import dataclasses

from .constants import AwsPartitionEnum


def _handle_empty_str(s: str) -> T.Optional[str]:
    if s:
        return s
    else:
        return None


def _handle_none(s: T.Optional[str]) -> str:
    if s is None:
        return ""
    else:
        return s


class AwsService:
    awslambda = "lambda"
    batch = "batch"
    cloudformation = "cloudformation"
    codebuild = "codebuild"
    codecommit = "codecommit"
    codepipeline = "codepipeline"
    dynamodb = "dynamodb"
    ec2 = "ec2"
    ecr = "ecr"
    ecs = "ecs"
    glue = "glue"
    iam = "iam"
    logs = "logs"
    rds = "rds"
    s3 = "s3"
    sagemaker = "sagemaker"
    secretsmanager = "secretsmanager"
    sns = "sns"
    sqs = "sqs"
    ssm = "ssm"
    states = "states"


class _Nothing:
    pass


NOTHING = _Nothing()


@dataclasses.dataclass
class BaseArn:
    """
    Amazon Resource Names (ARNs) data model. is a unique identifier for AWS resources.

    ARN format::

        - format: arn:${partition}:${service}:${region}:${account-id}:${resource-id}
            - example: arn:aws:sqs:us-east-1:111122223333:my-queue
        - format: arn:${partition}:${service}:${region}:${account-id}:${resource-type}${sep}${resource-id}
            - example sep = "/": arn:aws:iam::111122223333:role/aws-service-role/batch.amazonaws.com/AWSServiceRoleForBatch
        - format: arn:${partition}:${service}:${region}:${account-id}:${resource-type}${sep}${resource-id}
            - example sep = ":": arn:aws:batch:us-east-1:111122223333:job-definition/my-job-def:1

    Reference:

    - Amazon Resource Names (ARNs): https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html

    .. note::

        Don't initialize this class directly, use the class method :meth:`new` instead.
    """

    partition: str = dataclasses.field(default="aws")
    service: str = dataclasses.field(default=NOTHING)
    region: T.Optional[str] = dataclasses.field(default=NOTHING)
    account_id: T.Optional[str] = dataclasses.field(default=NOTHING)
    resource_type: T.Optional[str] = dataclasses.field(default=NOTHING)
    sep: T.Optional[str] = dataclasses.field(default=NOTHING)
    resource_id: str = dataclasses.field(default=NOTHING)

    def __post_init__(self):
        for k, v in dataclasses.asdict(self).items():
            # print([k, v])
            if isinstance(v, _Nothing):
                raise ValueError(f"arg '{k}' is required")

    @staticmethod
    def _parse_slash_delimited_resource(service: str, resource: str):
        # "arn:aws:s3:::my-bucket/file.txt"
        if service in ["s3", "apigateway"]:
            sep = None
            resource_type, resource_id = None, resource
        # arn:aws:ssm:us-east-1:807388292768:parameter/path/to/my_param
        elif service in ["ssm"]:
            sep = "/"
            resource_type, resource_id = resource.split("/", 1)
            if resource.count("/") > 1:
                resource_id = f"/{resource_id}"
        else:
            sep = "/"
            resource_type, resource_id = resource.split("/", 1)
        return sep, resource_type, resource_id

    @staticmethod
    def _parse_colon_delimited_resource(service: str, resource: str):
        if service in ["sns"]:
            sep = None
            resource_type, resource_id = None, resource
        else:
            sep = ":"
            resource_type, resource_id = resource.split(":", 1)
        return sep, resource_type, resource_id

    @classmethod
    def from_arn(cls, arn: str):
        """
        parse arn string into Arn object.
        """
        if not arn.startswith("arn:"):
            raise ValueError(f"Invalid ARN: {arn!r}")

        _, partition, service, region, account_id, resource = arn.split(":", 5)

        if "/" in resource and ":" in resource:
            if resource.index("/") < resource.index(":"):
                (
                    sep,
                    resource_type,
                    resource_id,
                ) = cls._parse_slash_delimited_resource(service, resource)
            else:
                (
                    sep,
                    resource_type,
                    resource_id,
                ) = cls._parse_colon_delimited_resource(service, resource)
        elif "/" in resource:
            (
                sep,
                resource_type,
                resource_id,
            ) = cls._parse_slash_delimited_resource(service, resource)
        elif ":" in resource:
            (
                sep,
                resource_type,
                resource_id,
            ) = cls._parse_colon_delimited_resource(service, resource)
        else:
            sep = None
            resource_type, resource_id = None, resource

        return cls(
            partition=partition,
            service=service,
            region=_handle_empty_str(region),
            account_id=_handle_empty_str(account_id),
            resource_type=resource_type,
            sep=sep,
            resource_id=resource_id,
        )

    def to_arn(self) -> str:
        """
        convert Arn object into arn string.
        """
        if self.sep:
            if self.sep == "/" and self.resource_id.startswith("/"):
                resource = f"{self.resource_type}{self.resource_id}"
            else:
                resource = f"{self.resource_type}{self.sep}{self.resource_id}"
        else:
            resource = self.resource_id
        return f"arn:{self.partition}:{self.service}:{_handle_none(self.region)}:{_handle_none(self.account_id)}:{resource}"

    @property
    def aws_account_id(self) -> T.Optional[str]:
        return self.account_id

    @property
    def aws_region(self) -> T.Optional[str]:
        return self.region

    def with_us_gov_partition(self):
        self.partition = AwsPartitionEnum.aws_us_gov.value
        return self

    def with_cn_partition(self):
        self.partition = AwsPartitionEnum.aws_cn.value
        return self

    def to_console_url(self) -> str: # pragma: no cover
        """
        AWS console has a magic url that can be used to open a resource in the console.
        However, this is undocumented, and only support a few services.
        """
        if self.aws_region:
            return f"https://{self.aws_region}.console.{self.partition}.amazon.com/go/view?arn={self.to_arn()}"
        else:
            return f"https://console.{self.partition}.amazon.com/go/view?arn={self.to_arn()}"


def is_arn_instance(obj: T.Any) -> bool:
    """
    Identify if an object is an instance of Arn.
    """
    return isinstance(obj, BaseArn)


@dataclasses.dataclass
class Arn(BaseArn):
    """
    todo: docstring
    """
    @classmethod
    def new(
        cls,
        service: str,
        partition: str,
        region: T.Optional[str],
        account_id: T.Optional[str],
        resource_type: T.Optional[str],
        sep: T.Optional[str],
        resource_id: str,
    ):  # pragma: no cover
        """
        Factory method.
        """
        return cls(
            service=service,
            partition=partition,
            region=region,
            account_id=account_id,
            resource_type=resource_type,
            sep=sep,
            resource_id=resource_id,
        )


@dataclasses.dataclass
class _CrossAccountGlobal(BaseArn):
    """
    No account, no region. Example:

    - AWS S3
        - Bucket: arn:aws:s3:::my-bucket
            - ${partition}: aws
            - ${service}: s3
            - ${region}: None
            - ${account-id}: None
            - ${resource-type}: None
            - ${sep}: None
            - ${resource-id}: my-bucket
    """

    region: T.Optional[str] = dataclasses.field(default=None)
    account_id: T.Optional[str] = dataclasses.field(default=None)


@dataclasses.dataclass
class _Global(BaseArn):
    """
    No region. Example:

    - AWS IAM:
        - Role: arn:aws:iam::807388292768:role/acu_e5f245a1_test
            - ${partition}: aws
            - ${service}: iam
            - ${region}: None
            - ${account-id}: 807388292768
            - ${resource-type}: role
            - ${sep}: "/"
            - ${resource-id}: acu_e5f245a1_test
    - AWS Route53
    """

    region: T.Optional[str] = dataclasses.field(default=None)


@dataclasses.dataclass
class _Regional(BaseArn):
    """
    Normal regional resources. Example:

    - AWS SQS
    - AWS Lambda

    .. warning::

        Don't subclass this class directly, use one of those:

        - :class:`ResourceIdOnlyRegional`
        - :class:`ColonSeparatedRegional`
        - :class:`SlashSeparatedRegional`
    """


@dataclasses.dataclass
class _ResourceIdOnlyRegional(_Regional):
    """
    Only one resource type in this service. Example:

    - AWS SQS:
        - queue: arn:aws:sqs:us-east-1:807388292768:acu_e5f245a1_test
            - ${partition}: aws
            - ${service}: sqs
            - ${region}: us-east-1
            - ${account-id}: 807388292768
            - ${resource-type}: None
            - ${sep}: None
            - ${resource-id}: acu_e5f245a1_test
    - AWS SNS:
        - queue: arn:aws:sns:us-east-1:807388292768:acu_e5f245a1_test
            - ${partition}: aws
            - ${service}: sns
            - ${region}: us-east-1
            - ${account-id}: 807388292768
            - ${resource-type}: None
            - ${sep}: None
            - ${resource-id}: acu_e5f245a1_test
    """

    resource_type: T.Optional[str] = dataclasses.field(default=None)
    sep: T.Optional[str] = dataclasses.field(default=None)


@dataclasses.dataclass
class _ColonSeparatedRegional(_Regional):
    """
    Example:

    - AWS Lambda:
        - Function: arn:aws:lambda:us-east-1:807388292768:function:acu_e5f245a1_test
            - ${partition}: aws
            - ${service}: lambda
            - ${region}: us-east-1
            - ${account-id}: 807388292768
            - ${resource-type}: function
            - ${sep}: ":"
            - ${resource-id}: acu_e5f245a1_test
        - Version: arn:aws:lambda:us-east-1:807388292768:function:acu_e5f245a1_test:1
            - ${resource-id}: acu_e5f245a1_test:1
        - Alias: arn:aws:lambda:us-east-1:807388292768:function:acu_e5f245a1_test:LIVE
            - ${resource-id}: acu_e5f245a1_test:LIVE
    """

    sep: T.Optional[str] = dataclasses.field(default=":")


@dataclasses.dataclass
class _SlashSeparatedRegional(_Regional):
    """
    Example:

    - AWS CloudFormation
        - Stack: arn:aws:cloudformation:us-east-1:807388292768:stack/CDKToolkit/d8677750-1d2c-11ee-b9bb-0e26849ff9df
            - ${partition}: aws
            - ${service}: cloudformation
            - ${region}: us-east-1
            - ${account-id}: 807388292768
            - ${resource-type}: stack
            - ${sep}: "/"
            - ${resource-id}: CDKToolkit/d8677750-1d2c-11ee-b9bb-0e26849ff9df
        - StackSet: arn:aws:cloudformation:us-east-1:807388292768:stackset/acu-ab1049bd-test-self-managed:273c6018-4440-40c5-8869-89c3e4b17f84
            - ${resource-type}: stackset
            - ${resource-id}: acu-ab1049bd-test-self-managed:273c6018-4440-40c5-8869-89c3e4b17f84
    """

    sep: T.Optional[str] = dataclasses.field(default="/")
